@@ -202,6 +202,7 @@ main(argc, argv)
 				sprintf(pooption, "-P%s", optarg+1);
 				postoffice = pooption + 2 /* "-P" */;
 				break;
+			case 'd':	/* -odb -- ignore quietly */
 			case 'b':	/* -obd -- ignore quietly */
 				break;
 			case 'e':	/* -oem -- delivery mail, errors to sender.
@@ -692,7 +693,7 @@ check_and_print_to(mfp, addr, notify)
 	char *addr, *notify;
 {
 	char *copy = NULL, *printme = addr, *from, *to;
-	char *s, newcp = NULL;
+	char *s, *newcp = NULL;
 
 	if (addr == NULL) {
 	  fprintf(stderr, "sendmail: Argument botch: NULL address\n");
