@@ -1,6 +1,9 @@
 /*
  *	Copyright 1989 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
+ *
+ *	Fully LISPis memory allocation scheme with garbage collection
+ *	by   Matti Aarnio <mea@nic.funet.fi> 1996
  */
 
 #include "malloc.h"
@@ -21,16 +24,25 @@ struct conscell {
 		struct conscell *u_dtpr;
 		unsigned char	*u_string;
 	} u;
+#ifdef __alpha	/* Alpha doesn't take nicely stores into too small objects.
+		   The hardware has store support only for 32/64/128-bit
+		   objects. */
+	int		flags;	/* if 0 is a list, otherwise a string */
+	int		pflags;	/* if 1 x->prev->cdr == x */
+#else
 	short		flags;	/* if 0 is a list, otherwise a string */
 	short		pflags;	/* if 1 x->prev->cdr == x */
+#endif
 	struct conscell *prev;	/* points to whichever dtpr points to here */
 };
 
-#define	NEWSTRING	001	/* newly allocated string */
-#define	CONSTSTRING	002	/* constant, mutually exclusive with above */
-#define	QUOTEDSTRING	004	/* quoted, don't apply IFS after expansion */
-#define	NOQUOTEIFQUOTED	010	/* if to be quoted, don't quote (for "$@") */
-#define ELEMENT		020	/* result of element expansion */
+#define	NEWSTRING	0001	/* newly allocated string (free() when GC) */
+#define	CONSTSTRING	0002	/* constant, mutually exclusive with above */
+#define	QUOTEDSTRING	0004	/* quoted, don't apply IFS after expansion */
+#define	NOQUOTEIFQUOTED	0010	/* if to be quoted, don't quote (for "$@") */
+#define ELEMENT		0020	/* result of element expansion */
+#define DSW_MARKER	0040	/* for garbage collection run.. */
+#define DSW_BACKPTR	0100	/* ... this too (DSW == Deutch-Schorr-Waite) */
 
 #define LIST(C)		((C)->flags == 0)
 #define STRING(C)	((C)->flags != 0)
@@ -63,7 +75,7 @@ struct conscell {
 
 /* YOU MUST BE VERY CAREFUL ABOUT CALLING MACROS FROM WITHIN MACROS!!! */
 /* BE VERY CAREFUL ABOUT USING EXPRESSIONS WITH SIDEEFFECTS INSIDE MACRO ARGS */
-#define newcell()	(struct conscell *)tmalloc(sizeof (struct conscell))
+/* #define newcell()	(struct conscell *)tmalloc(sizeof (struct conscell)) */
 #define copycell(X)	(tmp = newcell(), *tmp = *(X), tmp)
 /* nconc(list, list) -> old (,@list ,@list) */
 #define nconc(X,Y)	(car(X) != 0 ? cdr(s_last(car(X))) = (Y) \
@@ -106,6 +118,11 @@ extern struct conscell *s_copy_tree();
 extern void s_free_tree();
 #endif	/* MALLOC_TRACE */
 
+/* LISPic memory allocator, and other stuff.. */
+extern struct conscell * newcell();
+extern int cons_garbage_collect();
+extern int consvar_register(/* const struct conscell* */);
+extern void *consvar_mark();
+extern void consvar_release(/* void* */);
+
 #endif	/* _LISTUTILS_H */
-
-
