@@ -147,9 +147,10 @@ extern void	       diagnostic __((struct rcpt *rp, int rc, int timeout, char *fm
 extern void	       diagnostic __((/* struct rcpt *, int, int, char *,... */));
 #endif
 
-/* dnsgetrr.c */
-extern int	getrr     __((char *, int, int, int, FILE *));
-extern int	getrrtype __((char *, int, int, int, FILE *));
+
+#ifdef HOST_NOT_FOUND /* Defines 'struct hostent' ... */
+# include "dnsgetrr.h"
+#endif
 
 /* emptyline.c: */
 extern int	       emptyline __((char *line, int size));
