@@ -35,6 +35,8 @@ extern int errno;
 #include <sys/select.h>
 #endif
 
+#include <setjmp.h>
+
 #ifndef FNDELAY
 #define FNDELAY O_NDELAY
 #endif
@@ -43,7 +45,254 @@ unsigned short ident_tcpport = 113;
 
 #define SIZ 500 /* various buffers */
 
+
+#define CLORETS(e) { saveerrno = errno; close(s); errno = saveerrno; return e; }
+
+
+static jmp_buf jmpalarm;
+
+static int ident_tcpsock5(af, len, inlocal, inremote, sockp)
+int af, len;
+register int *inlocal;
+register int *inremote;
+int *sockp;
+{
+  union {
+    struct sockaddr_in  v4;
+#ifdef AF_INET6
+    struct sockaddr_in6 v6;
+#endif
+  } sa;
+  int addrlen;
+  register int s;
+  register int fl;
+  register int saveerrno;
+
+  *sockp = -1;
+
+  if ((s = socket(af, SOCK_STREAM, 0)) == -1)
+    return -1;
+
+  memset(&sa,0,sizeof(sa));
+  if (af == AF_INET) {
+    sa.v4.sin_family = AF_INET;
+    sa.v4.sin_port = 0;
+    memcpy(&sa.v4.sin_addr, inlocal, len);
+    if (sa.v4.sin_addr.s_addr != 0)
+      if (bind(s,(struct sockaddr*)&sa,sizeof(sa.v4)) < 0)
+	CLORETS(-1);
+  }
+#ifdef AF_INET6
+  else if (af == AF_INET6) {
+    sa.v6.sin6_family   = AF_INET6;
+    sa.v6.sin6_flowinfo = 0;
+    sa.v6.sin6_port     = 0;
+    memcpy(&sa.v6.sin6_addr, inlocal, len);
+
+    if (bind(s, (struct sockaddr*)&sa, sizeof(sa.v6)) < 0)
+      CLORETS(-1);
+  }
+#endif
+  else {
+    CLORETS(-1);
+  }
+
+  if ((fl = fcntl(s, F_GETFL, 0)) == -1)
+    CLORETS(-1);
+  if (fcntl(s, F_SETFL, FNDELAY | fl) == -1)
+    CLORETS(-1);
+
+  *sockp = s; /* Following connect may take non-trivial time,
+		 and we may be jumping around with SIGALRM,
+		 so for now do save the socket number.. */
+
+  memset(&sa,0,sizeof(sa));
+  if (af == AF_INET) {
+    sa.v4.sin_family  = AF_INET;
+    sa.v4.sin_port    = htons(ident_tcpport);
+    memcpy(&sa.v4.sin_addr, inremote, len);
+    if (connect(s,(struct sockaddr*)&sa,sizeof(sa.v4)) < 0)
+      if (errno != EINPROGRESS)
+	CLORETS(-1);
+  }
+#ifdef AF_INET6
+  else if (af == AF_INET6) {
+    sa.v6.sin6_family   = AF_INET6;
+    sa.v6.sin6_flowinfo = 0;
+    sa.v6.sin6_port     = htons(ident_tcpport);
+    memcpy(&sa.v6.sin6_addr, inremote, len);
+    if (connect(s,(struct sockaddr*)&sa,sizeof(sa.v6)) < 0)
+      if (errno != EINPROGRESS)
+	CLORETS(-1);
+  }
+#endif
+  else {
+    CLORETS(-1);
+  }
+  return s;
+}
+
+
+char *ident_sockuser2(s,local,remote,realbuf,realbuflen)
+int s;
+unsigned short local;
+unsigned short remote;
+char *realbuf;
+int realbuflen;
+{
+  register int buflen;
+  register int w;
+  register int saveerrno;
+  char ch;
+  unsigned short rlocal;
+  unsigned short rremote;
+  register int fl;
+  fd_set wfds;
+  void (*old_sig)__((int));
+  char *buf;
+ 
+  old_sig = signal(SIGPIPE, SIG_IGN);
+ 
+  FD_ZERO(&wfds);
+  FD_SET(s,&wfds);
+  select(s + 1,(fd_set *) 0,&wfds,(fd_set *) 0,(struct timeval *) 0);
+  /* now s is writable */
+  if ((fl = fcntl(s,F_GETFL,0)) == -1) {
+    signal(SIGPIPE, old_sig);
+    CLORETS("SOCKFCNTL1");
+  }
+  if (fcntl(s,F_SETFL,~FNDELAY & fl) == -1) {
+    signal(SIGPIPE, old_sig);
+    CLORETS("SOCKFCNTL2");
+  }
+  buf = realbuf;
+  sprintf(buf,"%u , %u\r\n",(unsigned int) remote,(unsigned int) local);
+  /* note the reversed order---the example in RFC 931 is misleading */
+  buflen = strlen(buf);
+  while ((w = write(s,buf,buflen)) < buflen)
+    if (w == -1) /* should we worry about 0 as well? */ {
+      signal(SIGPIPE, old_sig);
+      saveerrno = errno;
+      close(s);
+      if (errno == ECONNREFUSED)
+	return "NO-IDENT-SERVICE";
+      else
+	return "SOCKWRITE";
+    } else {
+      buf += w;
+      buflen -= w;
+    }
+  buf = realbuf;
+  while ((w = read(s,&ch,1)) == 1) {
+    *buf = ch;
+    if ((ch != ' ') && (ch != '\t') && (ch != '\r'))
+      ++buf;
+    if (((buf - realbuf) > (realbuflen -1) ) || (ch == '\n'))
+      break;
+  }
+  signal(SIGPIPE, old_sig);
+  if (w == -1)
+    CLORETS("SOCKREAD");
+  *buf = 0;
+
+  if (sscanf(realbuf,"%hd,%hd: USERID :%*[^:]:%s",&rremote,&rlocal,realbuf) < 3) {
+    close(s);
+    errno = EIO;
+    /* makes sense, right? well, not when USERID failed to match ERROR */
+    /* but there's no good error to return in that case */
+    
+    return "IDENT-NONSENSE";
+  }
+  if ((remote != rremote) || (local != rlocal)) {
+    close(s);
+    errno = EIO;
+    return "IDENT-NONSENSE2";
+  }
+  /* we're not going to do any backslash processing */
+  close(s);
+  return realbuf;
+}
+
+
+char *ident_tcpuser9(af,len,inlocal,inremote,local,remote,timeout,buf,buflen)
+	int af, len;
+	int *inlocal;
+	int *inremote;
+	unsigned short local;
+	unsigned short remote;
+	int timeout;
+	char *buf;
+	int buflen;
+{
+  int s, r;
+  struct timeval tv;
+  fd_set wfds;
+  int saveerrno;
+  void (*old_sig)__((int));
+  char *retval;
+  
+  old_sig = signal(SIGPIPE, SIG_IGN);
+  s = -1;
+
+  r = ident_tcpsock5(af, len, inlocal, inremote, &s);
+  if (r < 0) {
+    signal(SIGPIPE, old_sig);
+    return "SOCKFAULT1";
+  }
+  tv.tv_sec = timeout;
+  tv.tv_usec = 0;
+  FD_ZERO(&wfds);
+  FD_SET(s,&wfds);
+  r = select(s + 1, NULL, &wfds, NULL,&tv);
+  /* XXX: how to handle EINTR? */
+  if (r == -1) {
+    signal(SIGPIPE, old_sig);
+    CLORETS("SOCKSELERR");
+  }
+  if (!FD_ISSET(s,&wfds)) {
+    close(s);
+    errno = ETIMEDOUT;
+    signal(SIGPIPE, old_sig);
+    return "TIMEDOUT";
+  }
+  retval = ident_sockuser2(s,local,remote,buf,buflen);
+  signal(SIGPIPE, old_sig);
+  
+  return retval;
+}
+
+
 #if 0
+/* ------- various unused things! ---------------*/
+
+int ident_fd2(fd,inlocal,inremote,local,remote)
+register int fd;
+register struct in_addr *inlocal;
+register struct in_addr *inremote;
+register unsigned short *local;
+register unsigned short *remote;
+{
+  struct sockaddr_in sa;
+  int dummy;
+
+  dummy = sizeof(sa);
+  if (getsockname(fd,(struct sockaddr*)&sa,&dummy) == -1)
+    return 1;
+  if (sa.sin_family != AF_INET) {
+    errno = EAFNOSUPPORT;
+    return 2;
+  }
+  *local = ntohs(sa.sin_port);
+  *inlocal = sa.sin_addr;
+  dummy = sizeof(sa);
+  if (getpeername(fd,(struct sockaddr*)&sa,&dummy) == -1)
+    return 3;
+  *remote = ntohs(sa.sin_port);
+  *inremote = sa.sin_addr;
+  return 0;
+}
+
+
 static int usercmp(u,v)
 register char *u;
 register char *v;
@@ -246,10 +495,6 @@ register unsigned short remote;
   close(s);
   return ruser;
 }
-#endif
-
-
-#define CLORETS(e) { saveerrno = errno; close(s); errno = saveerrno; return e; }
 
 int ident_tcpsock(inlocal,inremote)
 register struct in_addr *inlocal;
@@ -284,160 +529,4 @@ register struct in_addr *inremote;
       CLORETS(-1);
   return s;
 }
-
-
-char *ident_sockuser2(s,local,remote,realbuf,realbuflen)
-int s;
-unsigned short local;
-unsigned short remote;
-char *realbuf;
-int realbuflen;
-{
-  register int buflen;
-  register int w;
-  register int saveerrno;
-  char ch;
-  unsigned short rlocal;
-  unsigned short rremote;
-  register int fl;
-  fd_set wfds;
-  void (*old_sig)__((int));
-  char *buf;
- 
-  old_sig = signal(SIGPIPE, SIG_IGN);
- 
-  FD_ZERO(&wfds);
-  FD_SET(s,&wfds);
-  select(s + 1,(fd_set *) 0,&wfds,(fd_set *) 0,(struct timeval *) 0);
-  /* now s is writable */
-  if ((fl = fcntl(s,F_GETFL,0)) == -1) {
-    signal(SIGPIPE, old_sig);
-    CLORETS("SOCKFCNTL1");
-  }
-  if (fcntl(s,F_SETFL,~FNDELAY & fl) == -1) {
-    signal(SIGPIPE, old_sig);
-    CLORETS("SOCKFCNTL2");
-  }
-  buf = realbuf;
-  sprintf(buf,"%u , %u\r\n",(unsigned int) remote,(unsigned int) local);
-  /* note the reversed order---the example in RFC 931 is misleading */
-  buflen = strlen(buf);
-  while ((w = write(s,buf,buflen)) < buflen)
-    if (w == -1) /* should we worry about 0 as well? */ {
-      signal(SIGPIPE, old_sig);
-      saveerrno = errno;
-      close(s);
-      if (errno == ECONNREFUSED)
-	return "NO-IDENT-SERVICE";
-      else
-	return "SOCKWRITE";
-    } else {
-      buf += w;
-      buflen -= w;
-    }
-  buf = realbuf;
-  while ((w = read(s,&ch,1)) == 1) {
-    *buf = ch;
-    if ((ch != ' ') && (ch != '\t') && (ch != '\r'))
-      ++buf;
-    if (((buf - realbuf) > (realbuflen -1) ) || (ch == '\n'))
-      break;
-  }
-  signal(SIGPIPE, old_sig);
-  if (w == -1)
-    CLORETS("SOCKREAD");
-  *buf = 0;
-
-  if (sscanf(realbuf,"%hd,%hd: USERID :%*[^:]:%s",&rremote,&rlocal,realbuf) < 3) {
-    close(s);
-    errno = EIO;
-    /* makes sense, right? well, not when USERID failed to match ERROR */
-    /* but there's no good error to return in that case */
-    
-    return "IDENT-NONSENSE";
-  }
-  if ((remote != rremote) || (local != rlocal)) {
-    close(s);
-    errno = EIO;
-    return "IDENT-NONSENSE2";
-  }
-  /* we're not going to do any backslash processing */
-  close(s);
-  return realbuf;
-}
-
-
-char *ident_tcpuser4(inlocal,inremote,local,remote,timeout,buf,buflen)
-struct in_addr *inlocal;
-struct in_addr *inremote;
-unsigned short local;
-unsigned short remote;
-int timeout;
-char *buf;
-int buflen;
-{
-  register int s;
-  struct timeval tv;
-  fd_set wfds;
-  register int r;
-  register int saveerrno;
-  void (*old_sig)__((int));
-  char *retval;
- 
-
-  old_sig = signal(SIGPIPE, SIG_IGN);
- 
-  s = ident_tcpsock(inlocal,inremote);
-  if (s == -1) {
-    signal(SIGPIPE, old_sig);
-    return "SOCKFAULT1";
-  }
-  tv.tv_sec = timeout;
-  tv.tv_usec = 0;
-  FD_ZERO(&wfds);
-  FD_SET(s,&wfds);
-  r = select(s + 1,(fd_set *) 0,&wfds,(fd_set *) 0,&tv);
-  /* XXX: how to handle EINTR? */
-  if (r == -1) {
-    signal(SIGPIPE, old_sig);
-    CLORETS("SOCKSELERR");
-  }
-  if (!FD_ISSET(s,&wfds)) {
-    close(s);
-    errno = ETIMEDOUT;
-    signal(SIGPIPE, old_sig);
-    return "TIMEDOUT";
-  }
-  retval = ident_sockuser2(s,local,remote,buf,buflen);
-  signal(SIGPIPE, old_sig);
-  
-  return retval;
-}
-
-
-int ident_fd2(fd,inlocal,inremote,local,remote)
-register int fd;
-register struct in_addr *inlocal;
-register struct in_addr *inremote;
-register unsigned short *local;
-register unsigned short *remote;
-{
-  struct sockaddr_in sa;
-  int dummy;
-
-  dummy = sizeof(sa);
-  if (getsockname(fd,(struct sockaddr*)&sa,&dummy) == -1)
-    return 1;
-  if (sa.sin_family != AF_INET) {
-    errno = EAFNOSUPPORT;
-    return 2;
-  }
-  *local = ntohs(sa.sin_port);
-  *inlocal = sa.sin_addr;
-  dummy = sizeof(sa);
-  if (getpeername(fd,(struct sockaddr*)&sa,&dummy) == -1)
-    return 3;
-  *remote = ntohs(sa.sin_port);
-  *inremote = sa.sin_addr;
-  return 0;
-}
+#endif
