@@ -152,11 +152,12 @@ dprintf(msg, num)
 #endif
 
 static struct hostent *
-getanswer(answer, anslen, qname, qtype)
+getanswer(answer, anslen, qname, qtype, ttlp)
 	const querybuf *answer;
 	int anslen;
 	const char *qname;
 	int qtype;
+	int *ttlp;
 {
 	register const HEADER *hp;
 	register const u_char *cp;
@@ -170,6 +171,8 @@ getanswer(answer, anslen, qname, qtype)
 	const char *tname;
 	int (*name_ok) __P((const char *));
 
+	*ttlp = 0;
+
 	tname = qname;
 	host.h_name = NULL;
 	eom = answer->buf + anslen;
@@ -233,7 +236,9 @@ getanswer(answer, anslen, qname, qtype)
 		type = _getshort(cp);
  		cp += INT16SZ;			/* type */
 		class = _getshort(cp);
- 		cp += INT16SZ + INT32SZ;	/* class, TTL */
+ 		cp += INT16SZ;			/* class */
+		*ttlp = _getlong(cp);
+		cp += INT32SZ;			/* TTL */
 		n = _getshort(cp);
 		cp += INT16SZ;			/* len */
 		if (class != C_IN) {
@@ -424,10 +429,22 @@ gethostbyname(name)
 	return (gethostbyname2(name, AF_INET));
 }
 
+extern struct hostent * gethostbyname2wttl __((const char *, int, int*));
+
 struct hostent *
 gethostbyname2(name, af)
 	const char *name;
 	int af;
+{
+	int ttl;
+	return gethostbyname2wttl(name, af, &ttl);
+}
+
+struct hostent *
+gethostbyname2wttl(name, af, ttlp)
+	const char *name;
+	int af;
+	int *ttlp;
 {
 	querybuf buf;
 	register const char *cp;
@@ -509,13 +526,25 @@ gethostbyname2(name, af)
 			return (_gethtbyname2(name, af));
 		return (NULL);
 	}
-	return (getanswer(&buf, n, name, type));
+	return (getanswer(&buf, n, name, type, ttlp));
 }
 
+extern struct hostent * gethostbyaddrwttl __((const char *addr, int, int, int*));
+
 struct hostent *
 gethostbyaddr(addr, len, af)
 	const char *addr;	/* XXX should have been def'd as u_char! */
 	int len, af;
+{
+	int ttl;
+	return gethostbyaddrwttl(addr, len, af, &ttl);
+}
+
+struct hostent *
+gethostbyaddrwttl(addr, len, af, ttlp)
+	const char *addr;	/* XXX should have been def'd as u_char! */
+	int len, af;
+	int *ttlp;
 {
 	const u_char *uaddr = (const u_char *)addr;
 	static const u_char mapped[] = { 0,0, 0,0, 0,0, 0,0, 0,0, 0xff,0xff };
@@ -589,7 +618,7 @@ gethostbyaddr(addr, len, af)
 			return (_gethtbyaddr(addr, len, af));
 		return (NULL);
 	}
-	if (!(hp = getanswer(&buf, n, qbuf, T_PTR)))
+	if (!(hp = getanswer(&buf, n, qbuf, T_PTR, ttlp)))
 		return (NULL);	/* h_errno was set by getanswer() */
 #ifdef SUNSECURITY
 	if (af == AF_INET) {
