@@ -49,7 +49,7 @@ extern int sh_read(), sh_echo(), sh_read(), sh_test(), sh_include();
 extern int sh_cd(), sh_colon(), sh_hash(), sh_bce(), sh_eval(), sh_export();
 extern int sh_getopts(), sh_shift(), sh_umask(), sh_set(), sh_unset();
 extern int sh_wait(), sh_times(), sh_trap(), sh_type(), sh_builtin();
-extern int sh_sleep(), sh_expr();
+extern int sh_sleep(), sh_expr(), sh_true(), sh_false();
 
 
 extern char *ifs;
@@ -116,6 +116,8 @@ struct shCmd builtins[] = {
 {	"type",		sh_type,	0,		0		},
 {	"builtin",	sh_builtin,	0,		0		},
 {	"sleep",	sh_sleep,	0,		0		},
+{	"true",		sh_true,	0,		0		},
+{	"false",	sh_false,	0,		0		},
 {	0,		0,		0,		0		},
 };
 
@@ -1156,3 +1158,19 @@ sh_expr(argc,argv)
 	return 1;
 #endif
 }
+
+int
+sh_true(argc, argv)
+	int argc;
+	char *argv[];
+{
+	return 0;
+}
+
+int
+sh_false(argc, argv)
+	int argc;
+	char *argv[];
+{
+	return 1;
+}
