@@ -17,6 +17,8 @@
 #include <fcntl.h>
 #include <sys/file.h>
 
+#include "syslog.h"
+
 /* #define static
    #define register */
 #define RUNIO(X)	(/*fprintf(runiofp, "runio(%s,%d)\n", __FILE__, __LINE__), */runio(&X))
@@ -25,7 +27,6 @@
 #include "io.h"
 #include "shconfig.h"
 
-
 int magic_number = 3;		/* check id for precompiled script files */
 long bin_magic   = 1;		/* Another check-id -- exec file  st_ctime ? */
 
@@ -84,6 +85,7 @@ STATIC short	fds[MAXNFILE];			/* could be a bitmap */
  * set of file descriptors.
  */
 
+STATIC void freeio __((struct IOop *, int));
 STATIC void
 freeio(fioop, mark)
 	register struct IOop *fioop;
@@ -112,7 +114,7 @@ freeio(fioop, mark)
 }
 
 
-
+STATIC char *dequote __((char *));
 STATIC char *
 dequote (str)
 	char *str;
@@ -133,6 +135,7 @@ dequote (str)
 	return (strnsave(sp, len));
 }
 
+STATIC void free_regexp __((regexp *prog));
 STATIC void
 free_regexp (prog)
 	regexp *prog;
@@ -143,6 +146,7 @@ free_regexp (prog)
 	}
 }
 
+STATIC u_char *regsub __((regexp *, int));
 STATIC u_char *
 regsub(prog, n)
 	regexp *prog;
@@ -154,6 +158,7 @@ regsub(prog, n)
 	return (u_char*)(prog->match[n]);
 }
 
+STATIC regexp *reg_comp __((char *));
 STATIC regexp *
 reg_comp (str)
 	char	*str;
@@ -182,6 +187,7 @@ reg_comp (str)
 	return (prog);
 }
 
+STATIC int reg_exec __((regexp *, char *));
 STATIC int
 reg_exec (prog, str)
 	regexp	*prog;
@@ -251,6 +257,7 @@ reg_exec (prog, str)
  * Return the next available filedescriptor, simulating kernel lookup.
  */
 
+STATIC int findfreefd __((void));
 STATIC int
 findfreefd()
 {
@@ -598,10 +605,12 @@ ioop(cmd, command, name, ioflags, defaultfd, arg1)
  * The interpreter interface to the execute() routine.
  */
 
+STATIC void runcommand __((struct osCmd *, struct osCmd *, int *, char *));
 STATIC void
-runcommand(c, pc, retcodep)
+runcommand(c, pc, retcodep, cmdname)
 	struct osCmd *c, *pc;
 	int *retcodep;
+	char *cmdname;
 {
 	int ioflags;
 	u_char *name;
@@ -817,6 +826,7 @@ assign(sl_lhs, sl_rhs, command)
  * the last definition for the stored code table, free the table.
  */
 
+STATIC void undefun __((char *));
 STATIC void
 undefun(fname)
 	char *fname;
@@ -888,6 +898,7 @@ xundefun(spl)
  * descriptor and the code table descriptor.
  */
 
+STATIC void defun __((struct codedesc *, char *, u_char *, u_char *));
 STATIC void
 defun(cdp, fname, position, eofunc)
 	struct codedesc *cdp;
@@ -1142,6 +1153,14 @@ tscanstring(s)
  * data (which is malloc()'ed) whenever we discard it.
  */
 
+
+/* This is a last-line defence on utter stupidity -- doing recursive
+ * evaluation on something without proper terminating condition does
+ * quickly lead to awfull crash with a system whose stack is demolished,
+ * and rather difficult to debug...   Trust me, I know -- Matti Aarnio */
+
+static int max_interpreter_recursions = 40;
+
 struct codedesc *
 interpret(code, eocode, entry, caller, retcodep, cdp)
 	u_char	*code, *eocode, *entry;
@@ -1154,7 +1173,7 @@ interpret(code, eocode, entry, caller, retcodep, cdp)
 	register OutputTokens cmd;
 	register struct osCmd *command;
 	register int commandIndex, variableIndex, i;
-	u_char	*arg1, *name;
+	u_char	*arg1, *name, *cmdname;
 	char	*origlevel;
 	int	*iname;
 	int	argi1, dye, childpid, dollar, ioflags;
@@ -1179,6 +1198,11 @@ interpret(code, eocode, entry, caller, retcodep, cdp)
 
 #ifdef	MAILER
 	++funclevel;
+	if (funclevel > max_interpreter_recursions) {
+	  fprintf(stderr,"zmailer: interpret: recursed more than %d levels deep on invocation!  script termination condition error ?\n",max_interpreter_recursions);
+	  syslog(LOG_EMERG,"zmailer: interpret: recursed more than %d levels deep on invocation!  script termination condition error ?\n",max_interpreter_recursions);
+	  abort(); /* excessively deep recursion - *.cf -script termination condition error ? */
+	}
 #endif	/* MAILER */
 	optind = 0;	/* for getopts */
 	if (cdp == NULL) {
@@ -1414,9 +1438,14 @@ interpret(code, eocode, entry, caller, retcodep, cdp)
 					 * ... else we're connecting two
 					 * list-valued functions, which is ok.
 					 */
+					cmdname = NULL;
+					if (prevcommand->sfdp)
+					  cmdname = prevcommand->sfdp->name;
+					if (prevcommand->shcmdp)
+					  cmdname = prevcommand->shcmdp->name;
 					runcommand(prevcommand, caller,
 						   ignore_level>commandIndex ?
-						   retcodep : (int *)NULL);
+						   retcodep : NULL, cmdname);
 					command->rval = prevcommand->rval;
 					if (command->prev == prevcommand) {
 						command->prev = NULL;
@@ -1636,9 +1665,14 @@ interpret(code, eocode, entry, caller, retcodep, cdp)
 			}
 			argi1 = *retcodep;
 			if (command->iocmd != ioCarryBuffer) {
+				cmdname = NULL;
+				if (command->sfdp)
+				  cmdname = command->sfdp->name;
+				if (command->shcmdp)
+				  cmdname = command->shcmdp->name;
 				runcommand(command, caller,
-					ignore_level > commandIndex ?
-					retcodep : (int *)NULL);
+					   ignore_level > commandIndex ?
+					   retcodep : NULL, cmdname);
 				tmp = *(command->bufferp);
 			}
 			if (ignore_level == commandIndex)
@@ -2029,9 +2063,14 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 				 * ... else we're connecting two
 				 * list-valued functions, which is ok.
 				 */
+				cmdname = NULL;
+				if (prevcommand->sfdp)
+				  cmdname = prevcommand->sfdp->name;
+				if (prevcommand->shcmdp)
+				  cmdname = prevcommand->shcmdp->name;
 				runcommand(prevcommand, caller,
 					   ignore_level > commandIndex ?
-					   retcodep : (int *)NULL);
+					   retcodep : NULL, cmdname);
 				command->rval = prevcommand->rval;
 				if (command->prev == prevcommand)
 					command->prev = NULL;
