@@ -948,9 +948,10 @@ dbtype(dbname)
  * The lookup sequence for foo.bar.edu is:
  *
  *	foo.bar.edu
+ *	.foo.bar.edu
  *	.bar.edu
  *	.edu
- *	.foo.bar.edu
+ *	.
  */
 
 static struct conscell *
@@ -960,24 +961,48 @@ find_domain(lookupfn, sip)
 {
 	register u_char *cp;
 	struct conscell *l;
-	u_char buf[BUFSIZ], *realkey;
+	u_char *realkey, *buf;
+	int keylen;
 
 	/* check the key as given */
 	if ((l = (*lookupfn)(sip)) != NULL)
 		return l;
+
+	realkey = sip->key;
+	keylen = strlen((char*)realkey);
+#if 1
+#ifdef HAVE_ALLOCA
+	buf = (u_char*)alloca(keylen+2);
+#else
+	buf = (u_char*)emalloc(keylen+2);
+#endif
+	/* No exact match, see if you can find a  "." + keystring ? */
+	if (*realkey != '.') {
+		buf[0] = '.';
+		strcpy((char *)&buf[1], (char *)realkey);
+	}
+#else
+	buf = realkey;
+#endif
 	/* iterate over the subdomains of the key */
-	for (cp = realkey = sip->key; *cp;) {
+	for (cp = buf; *cp;) {
 		while (*cp && *cp != '.')
 			++cp;
 		while (*cp == '.')
 			++cp;
 		if (*(cp-1) == '.') {
 			sip->key = cp-1;
-			if ((l = (*lookupfn)(sip)) != NULL)
+			if ((l = (*lookupfn)(sip)) != NULL) {
+#if 1
+#ifndef HAVE_ALLOCA
+				free(buf);
+#endif
+#endif
 				return l;
+			}
 		}
 	}
-#if 1
+#if 0
 	/* if all else failed, try prepending a dot and look for subdomains */
 	if (*realkey != '.' && strlen((char *)realkey) < sizeof buf - 1) {
 		buf[0] = '.';
