@@ -11,6 +11,9 @@
 #undef	RFC974		/* MX/WKS/A processing according to RFC974 */
 
 #include <netdb.h>
+#ifndef EAI_AGAIN
+# include "netdb6.h" /* IPv6 API stuff */
+#endif
 
 /* ================ do we have a normal nameserver? ================ */
 #ifdef	TRY_AGAIN
@@ -33,6 +36,10 @@
 #include <resolv.h>
 #include "search.h"
 
+#include "libc.h"
+#include "dnsgetrr.h"
+
+
 /*
  * Query a BIND standard (version 4.7 or later) nameserver.
  */
@@ -94,6 +101,9 @@ typedef char msgdata;
 #ifndef T_NSAP_PTR
 # define T_NSAP_PTR 23	/* reverse NSAP lookup (depreciated) */
 #endif
+#ifndef T_AAAA
+# define T_AAAA 28	/* IPv6 Address */
+#endif
 #ifndef	T_UINFO
 # define T_UINFO 100
 #endif
@@ -124,6 +134,7 @@ struct qtypes {
 	{	"any",		T_ANY		},
 	{	"mx",		T_MX		},
 	{	"a",		T_A		},
+	{	"aaaa",		T_AAAA		},
 #ifdef	T_MP
 	{	"mp",		T_MP		},
 #endif	/* T_MP */
@@ -145,7 +156,7 @@ char *zh_errlist[] = {
 	"no address associated with name"	/* 4 - NO_ADDRESS	*/
 };
 
-extern struct conscell *getmxrr(), *getcrrtype(), *getrrtype();
+extern struct conscell *getmxrr(), *getcrrtype(), *getrrtypec();
 
 extern char *conffile, *myhostname;
 extern int deferit, res_init(), cistrcmp();
@@ -229,7 +240,7 @@ search_res(sip)
 #endif	/* T_UNAME */
 	case T_TXT:
 	case T_UINFO:
-		rval = getrrtype(sip->key, qtp->value, &sip->ttl, 0);
+		rval = getrrtypec(sip->key, qtp->value, &sip->ttl, 0);
 		break;
 	case T_ANY:
 	case T_CNAME:
@@ -350,7 +361,7 @@ getmxrr(host, localhost, ttlp, depth)
 #endif	/* OLDJEEVES */
 			case NOERROR:
 				/* if we got this, then ancount == 0! */
-				return NULL /*getrrtype(host, T_A, ttlp)*/;
+				return NULL /*getrrtypec(host, T_A, ttlp)*/;
 #ifndef	OLDJEEVES
 			case FORMERR:
 #endif	/* !OLDJEEVES */
@@ -432,7 +443,7 @@ getmxrr(host, localhost, ttlp, depth)
 		if (mx[n].host == NULL)
 			continue;
 		strcpy(hbuf, mx[n].host);
-		if (!getrrtype(hbuf, T_WKS, ttlp, 0)) {
+		if (!getrrtypec(hbuf, T_WKS, ttlp, 0)) {
 			free(mx[n].host);
 			mx[n].host = NULL;
 		}
@@ -467,7 +478,7 @@ getmxrr(host, localhost, ttlp, depth)
 }
 
 struct conscell *
-getcrrtype(host, rrtype, ttlp, depth)	/* getrrtype with completion */
+getcrrtype(host, rrtype, ttlp, depth)	/* getrrtypec() with completion */
 	char *host;
 	u_short rrtype;
 	time_t *ttlp;
@@ -487,7 +498,7 @@ getcrrtype(host, rrtype, ttlp, depth)	/* getrrtype with completion */
 	}
 
 	if (strchr(host, '.') != NULL) {
-		if ((rval = getrrtype(host, rrtype, ttlp, depth+1)) != NULL)
+		if ((rval = getrrtypec(host, rrtype, ttlp, depth+1)) != NULL)
 			return rval;
 		if (*host != '\0' && *(host+strlen(host)-1) == '.')
 			return NULL;
@@ -495,7 +506,7 @@ getcrrtype(host, rrtype, ttlp, depth)	/* getrrtype with completion */
 	for (i = 0, domain = _res.dnsrch[i];
 	     h_errno == 0 && (domain = _res.dnsrch[i]) != NULL; ++i) {
 		sprintf(buf, "%s.%s", host, domain);
-		if ((rval = getrrtype(buf, rrtype, ttlp, depth+1)) != NULL)
+		if ((rval = getrrtypec(buf, rrtype, ttlp, depth+1)) != NULL)
 			return rval;
 	}
 	strcpy(h_errhost, host);
@@ -503,7 +514,7 @@ getcrrtype(host, rrtype, ttlp, depth)	/* getrrtype with completion */
 }
 
 struct conscell *
-getrrtype(host, rrtype, ttlp, depth)
+getrrtypec(host, rrtype, ttlp, depth)
 	char *host;
 	u_short rrtype;
 	time_t *ttlp;
@@ -512,7 +523,8 @@ getrrtype(host, rrtype, ttlp, depth)
 	struct conscell *lhead, *l, *tmp;
 	char *s;
 	HEADER *hp;
-	msgdata *eom, *cp, *nextcp;
+	msgdata *eom;
+	const msgdata *cp, *nextcp;
 	querybuf buf, answer;
 	int qlen, n, ancount, qdcount, ok, first;
 	struct in_addr inaddr;
@@ -667,7 +679,27 @@ getrrtype(host, rrtype, ttlp, depth)
 			  }
 			}
 			continue;
+		case T_AAAA:
+			if (rrtype == T_ANY) {
+			  return newstring((u_char*)strsave(host));
+			}
+#ifdef AF_INET6
+			{
+			  char tb[80];
+			  const char *ss;
 
+			  ss = inet_ntop(AF_INET6, cp, tb, sizeof(tb));
+			  s = strsave(ss);
+			  if (lhead == NULL)
+			    lhead = l = newstring((u_char *)s);
+			  else {
+			    cdr(l) = newstring((u_char *)s);
+			    l = cdr(l);
+			  }
+			  *ttlp = maxttl;
+			}
+#endif
+			continue;
 		case T_A:
 		case T_SA:
 			if (rrtype == T_ANY) {
@@ -712,11 +744,10 @@ getrrtype(host, rrtype, ttlp, depth)
 			if (type == T_CNAME && rrtype != T_ANY &&
 			    !CISTREQ(nbuf, host))
 				/* chase it down */
-				getrrtype((char *)nbuf, rrtype, ttlp, depth+1);
+				getrrtypec((char *)nbuf, rrtype, ttlp, depth+1);
 			*ttlp = maxttl;
 			return newstring((u_char*)strsave((char*)nbuf));
 
-	/*	case T_AAAA:	IPv6 'A'-record */
 		case T_SOA:
 		case T_NS:
 		case T_RP:
@@ -737,7 +768,7 @@ getrrtype(host, rrtype, ttlp, depth)
 			    return newstring((u_char*)strsave(host));
 			}
 		default:
-			fprintf(stderr,"search_res: getrrtype: non-processed RR type for host='%s' query=%d, result=%d\n",host,rrtype,type);
+			fprintf(stderr,"search_res: getrrtypec: non-processed RR type for host='%s' query=%d, result=%d\n",host,rrtype,type);
 
 			break;
 		}
