@@ -27,7 +27,9 @@ extern int dbminit();
 extern int deferit;
 extern int delete();
 extern int store();
-extern void dbmclose();
+#ifndef dbmclose
+ extern void dbmclose();
+#endif
 extern void v_set();
 
 
