@@ -52,6 +52,7 @@ extern struct conscell	*search_gdbm	__((struct search_info	*sip));
 extern struct conscell	*search_header	__((struct search_info	*sip));
 extern struct conscell	*search_hosts	__((struct search_info	*sip));
 extern struct conscell	*search_ndbm	__((struct search_info	*sip));
+extern struct conscell	*search_btree	__((struct search_info	*sip));
 extern struct conscell	*search_res	__((struct search_info	*sip));
 extern struct conscell	*search_seq	__((struct search_info	*sip));
 extern struct conscell	*search_yp	__((struct search_info	*sip));
@@ -59,6 +60,7 @@ extern struct conscell	*search_yp	__((struct search_info	*sip));
 extern void	close_core	__((struct search_info	*sip));
 extern void	close_seq	__((struct search_info	*sip));
 extern void	close_ndbm	__((struct search_info	*sip));
+extern void	close_btree	__((struct search_info	*sip));
 extern void	close_dbm	__((struct search_info	*sip));
 extern void	close_gdbm	__((struct search_info	*sip));
 extern void	close_header	__((struct search_info	*sip));
@@ -66,12 +68,14 @@ extern void	close_header	__((struct search_info	*sip));
 extern int	add_core	__((struct search_info *sip, char *value));
 extern int	add_seq		__((struct search_info *sip, char *value));
 extern int	add_ndbm	__((struct search_info *sip, char *value));
+extern int	add_btree	__((struct search_info *sip, char *value));
 extern int	add_dbm		__((struct search_info *sip, char *value));
 extern int	add_gdbm	__((struct search_info *sip, char *value));
 extern int	add_header	__((struct search_info *sip, char *value));
 /* removes	*/
 extern int	remove_core	__((struct search_info	*sip));
 extern int	remove_ndbm	__((struct search_info	*sip));
+extern int	remove_btree	__((struct search_info	*sip));
 extern int	remove_dbm	__((struct search_info	*sip));
 extern int	remove_gdbm	__((struct search_info	*sip));
 extern int	remove_header	__((struct search_info	*sip));
@@ -80,17 +84,30 @@ extern void	print_core	__((struct search_info *sip, FILE *outfp));
 extern void	print_hosts	__((struct search_info *sip, FILE *outfp));
 extern void	print_seq	__((struct search_info *sip, FILE *outfp));
 extern void	print_ndbm	__((struct search_info *sip, FILE *outfp));
+extern void	print_btree	__((struct search_info *sip, FILE *outfp));
 extern void	print_dbm	__((struct search_info *sip, FILE *outfp));
 extern void	print_gdbm	__((struct search_info *sip, FILE *outfp));
 extern void	print_header	__((struct search_info *sip, FILE *outfp));
+extern void	print_yp	__((struct search_info *sip, FILE *outfp));
+/* counts	*/
+extern void	count_core	__((struct search_info *sip, FILE *outfp));
+extern void	count_seq	__((struct search_info *sip, FILE *outfp));
+extern void	count_ndbm	__((struct search_info *sip, FILE *outfp));
+extern void	count_btree	__((struct search_info *sip, FILE *outfp));
+extern void	count_dbm	__((struct search_info *sip, FILE *outfp));
+extern void	count_gdbm	__((struct search_info *sip, FILE *outfp));
+extern void	count_header	__((struct search_info *sip, FILE *outfp));
 /* owners	*/
 extern void	owner_core	__((struct search_info *sip, FILE *outfp));
 extern void	owner_seq	__((struct search_info *sip, FILE *outfp));
 extern void	owner_ndbm	__((struct search_info *sip, FILE *outfp));
+extern void	owner_btree	__((struct search_info *sip, FILE *outfp));
 extern void	owner_dbm	__((struct search_info *sip, FILE *outfp));
 extern void	owner_gdbm	__((struct search_info *sip, FILE *outfp));
 extern void	owner_header	__((struct search_info *sip, FILE *outfp));
+extern void	owner_yp	__((struct search_info *sip, FILE *outfp));
 /* modchecks	*/
 extern int	modp_seq	__((struct search_info	*sip));
 extern int	modp_ndbm	__((struct search_info	*sip));
+extern int	modp_btree	__((struct search_info	*sip));
 extern int	modp_gdbm	__((struct search_info	*sip));
