@@ -944,7 +944,7 @@ mkTrace(e)
 	
 	th = (struct header *)tmalloc(sizeof (struct header));
 	th->h_pname = "Received";
-	th->h_descriptor = &traceDesc;
+	th->h_descriptor = (struct headerinfo *)&traceDesc;
 	th->h_stamp = newHeader;
 	th->h_next = 0;
 	th->h_contents.r = (struct received *)tmalloc(sizeof (struct received));
