@@ -241,6 +241,87 @@ FILE *mfp;
 }
 
 
+static void
+writeheader(errfp, eaddr, no_error_reportp, deliveryform, boundary)
+     FILE *errfp;
+     char *eaddr;
+     int *no_error_reportp;
+     char *deliveryform;
+     char *boundary;
+{
+	FILE *fp;
+	char path[MAXPATHLEN], mpath[MAXPATHLEN];
+	char buf[BUFSIZ];
+
+	if (*eaddr == '|' /* Nobody's address starts with a pipe ? */ ||
+	    (*eaddr == '/' &&
+	     (strchr(eaddr,'@') == NULL ||
+	      strchr(eaddr,'=') == NULL /*Smells of X.400*/)))
+	  eaddr = "postmaster"; /* Paranoid, eh ? */
+	fprintf(errfp, "channel error\n");
+
+	if (!*no_error_reportp)
+	  printenvaddr(errfp,eaddr);
+	sprintf(path, "%s/%s/%s", mailshare, FORMSDIR,
+		deliveryform ? deliveryform : "delivery");
+	if ((fp = fopen(path, "r")) != NULL) {
+	  int inhdr = 1, hadsubj =0;
+	  buf[sizeof(buf)-1] = 0;
+	  while (fgets(buf,sizeof(buf)-1,fp) != NULL) {
+	    if (strncmp(buf,"HDR",3)==0) {
+	      continue;
+	    } else if (strncmp(buf,"ADR",3)==0) {
+	      if (*no_error_reportp)
+		*no_error_reportp = -1;
+	      pick_env_addr(buf+4,errfp);
+	    } else if (strncmp(buf,"SUB",3)==0) {
+	      continue;
+	    } else
+	      break;
+	  }
+	  fputs("env-end\n",errfp); /* Envelope ends */
+	  fseek(fp, (off_t)0, SEEK_SET); /* Rewind! Start building the report */
+
+	  if (*no_error_reportp == 0)
+	    fprintf(errfp, "To: %s\n", eaddr);
+	  else if (*no_error_reportp < 0)
+	    fprintf(errfp, "To: dummy:; (error trapped source)\n");
+	  while (fgets(buf,sizeof(buf)-1,fp) != NULL) {
+	    if (strncmp(buf,"HDR",3)==0) {
+	      fputs(buf+4,errfp);
+	    } else if (strncmp(buf,"ADR",3)==0) {
+	      fputs(buf+4,errfp);
+	    } else if (strncmp(buf,"SUB",3)==0) {
+	      hadsubj = 1;
+	      if (*no_error_reportp < 0) {
+		/* We modify the subject string... */
+		char *s = buf+4;
+		while (*s && *s != ':') ++s;
+		if (*s == ':') {
+		  ++s;
+		  if (*s) *s++ = 0;
+		  fprintf(errfp,"%s Double-fault: %s", buf+4, s);
+		} else {
+		  fputs("Subject: Double-fault: unknown delivery error\n",errfp);
+		}
+	      } else
+		fputs(buf+4,errfp);
+	    } else {
+	      if (inhdr) {
+		inhdr = 0;
+		fprintf(errfp,"MIME-Version: 1.0\n");
+		fprintf(errfp,"Content-Type: multipart/report; report-type=delivery-status;\n");
+		fprintf(errfp,"\tboundary=\"%s\"\n\n",boundary);
+		fprintf(errfp, "--%s\n", boundary);
+		fprintf(errfp, "Content-Type: text/plain\n");
+	      }
+	      fputs(buf,errfp);
+	    }
+	  } /* ... while() ends.. */
+	  fclose(fp);
+	}
+}
+
 /* called to process errors at sensible intervals */
 
 void
@@ -263,7 +344,7 @@ reporterrs(cfpi)
 	int no_error_report = 0;
 	int has_errors = 0;
 	int lastoffset;
-	char path[MAXPATHLEN], buf[BUFSIZ], mpath[MAXPATHLEN];
+	char path[MAXPATHLEN], mpath[MAXPATHLEN];
 
 	if (cfpi->haderror == 0)
 		return;
@@ -344,6 +425,7 @@ reporterrs(cfpi)
 	  if (*cp == ':') {
 	    /* This was a field called DNSRECIPIENT -- which
 	       was never used in actuality ... */
+	    ++cp;
 	    while (isascii(*cp) && (isdigit(*cp) || *cp == '-'))
 	      ++cp;
 	  }
@@ -425,73 +507,7 @@ reporterrs(cfpi)
 	  }
 
 	  if (!wroteheader) {
-	    if (*eaddr == '|' /* Nobody's address starts with a pipe ? */ ||
-		(*eaddr == '/' &&
-		 (strchr(eaddr,'@') == NULL ||
-		  strchr(eaddr,'=') == NULL /*Smells of X.400*/)))
-	      eaddr = "postmaster"; /* Paranoid, eh ? */
-	    fprintf(errfp, "channel error\n");
-
-	    if (!no_error_report)
-	      printenvaddr(errfp,eaddr);
-	    sprintf(path, "%s/%s/%s", mailshare, FORMSDIR,
-		    deliveryform ? deliveryform : "delivery");
-	    if ((fp = fopen(path, "r")) != NULL) {
-	      int inhdr = 1, hadsubj =0;
-	      buf[sizeof(buf)-1] = 0;
-	      while (fgets(buf,sizeof(buf)-1,fp) != NULL) {
-		if (strncmp(buf,"HDR",3)==0) {
-		  continue;
-		} else if (strncmp(buf,"ADR",3)==0) {
-		  if (no_error_report)
-		    no_error_report = -1;
-		  pick_env_addr(buf+4,errfp);
-		} else if (strncmp(buf,"SUB",3)==0) {
-		  continue;
-		} else
-		  break;
-	      }
-	      fputs("env-end\n",errfp); /* Envelope ends */
-	      fseek(fp, (off_t)0, SEEK_SET); /* Rewind! Start building the report */
-
-	      if (no_error_report == 0)
-		fprintf(errfp, "To: %s\n", eaddr);
-	      else if (no_error_report < 0)
-		fprintf(errfp, "To: dummy:; (error trapped source)\n");
-	      while (fgets(buf,sizeof(buf)-1,fp) != NULL) {
-		if (strncmp(buf,"HDR",3)==0) {
-		  fputs(buf+4,errfp);
-		} else if (strncmp(buf,"ADR",3)==0) {
-		  fputs(buf+4,errfp);
-		} else if (strncmp(buf,"SUB",3)==0) {
-		  hadsubj = 1;
-		  if (no_error_report < 0) {
-		    /* We modify the subject string... */
-		    char *s = buf+4;
-		    while (*s && *s != ':') ++s;
-		    if (*s == ':') {
-		      ++s;
-		      if (*s) *s++ = 0;
-		      fprintf(errfp,"%s Double-fault: %s", buf+4, s);
-		    } else {
-		      fputs("Subject: Double-fault: unknown delivery error\n",errfp);
-		    }
-		  } else
-		    fputs(buf+4,errfp);
-		} else {
-		  if (inhdr) {
-		    inhdr = 0;
-		    fprintf(errfp,"MIME-Version: 1.0\n");
-		    fprintf(errfp,"Content-Type: multipart/report; report-type=delivery-status;\n");
-		    fprintf(errfp,"\tboundary=\"%s\"\n\n",boundarystr);
-		    fprintf(errfp, "--%s\n", boundarystr);
-		    fprintf(errfp, "Content-Type: text/plain\n");
-		  }
-		  fputs(buf,errfp);
-		}
-	      } /* ... while() ends.. */
-	      fclose(fp);
-	    }
+	    writeheader(errfp, eaddr, &no_error_report, deliveryform, boundarystr);
 	    /* fprintf(errfp, "\n"); -- not needed ?? */
 	    wroteheader = 1;
 
@@ -524,6 +540,15 @@ reporterrs(cfpi)
 	    fprintf(errfp, "%s\n", cp);
 	  lockaddr(cfp->fd, NULL, *lp + 1, _CFTAG_NORMAL, _CFTAG_OK, mpath, "diagmsg");
 	}
+	/* End of all diagnostics message lines */
+
+	if (!wroteheader) {
+	  writeheader(errfp, eaddr, &no_error_report, deliveryform, boundarystr);
+	  /* fprintf(errfp, "\n"); -- not needed ?? */
+	  wroteheader = 1;
+
+	} /* End of failure list processing */
+
 	
 	fprintf(errfp, "\n");
 	fprintf(errfp, "--%s\n", boundarystr);
@@ -564,6 +589,8 @@ reporterrs(cfpi)
 
 	if ((fp = fopen(path, "r")) != NULL) {
 
+	  char buf[BUFSIZ];
+
 	  if (cfp->msgbodyoffset > 0 && headeridx > 0 ) {
 	    /* We have knowledge about the headers of errored email,
 	       use those headers on output ! */
@@ -592,6 +619,7 @@ reporterrs(cfpi)
 	  if (has_errors && !never_full_content &&
 	      (!cfp->dsnretmode || CISTREQN(cfp->dsnretmode,"FULL",4))) {
 	    /* Copy out the rest (=body) with somewhat more efficient method */
+
 	    while ((n = fread(buf, sizeof buf[0], sizeof buf, fp)) > 0)
 	      fwrite(buf, sizeof buf[0], n, errfp);
 	  }
