@@ -75,6 +75,8 @@ char	*rendezvous;
 char	*pidfile = PID_SCHEDULER;
 char	*mailshare;
 char	*logfn;
+char	*statusfn;
+FILE    *statuslog = NULL;
 int	slow_shutdown = 0;
 extern int readsockcnt; /* from transport.c: mux() */
  
@@ -156,6 +158,48 @@ extern void die __((int rc, char *reason));
 extern int  killprevious __((int signal, char *pidfile));
 extern void prversion __((char *name));
 
+static int loginitsched(sig)
+int sig;
+{
+	int flags;
+
+	if (logfn != NULL) {
+	  fflush(stdout);
+	  rewind(stdout);
+	  fflush(stderr);
+	  rewind(stderr);
+	  if (freopen(logfn, "a", stdout) != stdout
+	      || dup2(FILENO(stdout), FILENO(stderr)) < 0) {	/* sigh */
+	    /* XX: stderr might be closed at this point... */
+	    fprintf(stderr, "%s: cannot open log: %s, errno=%d\n", progname, logfn, errno);
+	    return -1;
+	  }
+#if	defined(F_SETFL) && defined(O_APPEND)
+	  flags = fcntl(FILENO(stdout), F_GETFL, 0);
+	  flags |= O_APPEND;
+	  fcntl(FILENO(stdout), F_SETFL, flags);
+#endif	/* F_SETFL */
+#if defined(F_SETFD)
+	  fcntl(FILENO(stdout), F_SETFD, 1); /* close-on-exec */
+#endif
+	  setvbuf(stdout, (char *)NULL, _IOLBF, 0);
+	  setvbuf(stderr, (char *)NULL, _IOLBF, 0);
+	}
+	if (statusfn != NULL && statuslog != NULL) {
+	  fflush(statuslog);
+	  if (freopen(statusfn, "a", statuslog) != statuslog) {
+	    fprintf(stderr,"%s: cannot open statuslog: %s, errno=%d\n", progname, statusfn, errno);
+	    return -1;
+	  }
+#if defined(F_SETFD)
+	  fcntl(FILENO(statuslog), F_SETFD, 1); /* close-on-exec */
+#endif
+	  setvbuf(statuslog, (char *)NULL, _IOLBF, 0);
+	}
+	SIGNAL_HANDLE(SIGHUP, (RETSIGTYPE (*)()) loginitsched);
+	return 0;
+}
+
 struct dirstatname {
 	struct stat st;
 	long ino;
@@ -178,7 +222,6 @@ static int syncweb __((struct dirqueue *dq, struct config_entry *ce));
 
 int global_maxkids = 1000;
 time_t now;
-FILE *statuslog = NULL;
 
 FILE *vfp_open(cfp)
 struct ctlfile *cfp;
@@ -301,7 +344,7 @@ main(argc, argv)
 
 	global_maxkids = resources_query_nofiles()-10;
 
-	postoffice = rendezvous = logfn = config = NULL;
+	postoffice = rendezvous = logfn = statusfn = config = NULL;
 	daemonflg = 1;
 	dlyverbose = 0;
 	syncstart = 0;
@@ -320,11 +363,17 @@ main(argc, argv)
 			freeze = 1;
 			break;
 		case 'l':
-			statuslog = fopen(optarg,"a");
+			statusfn = optarg;
+			statuslog = fopen(statusfn,"a");
 			if (!statuslog) {
 			  perror("Can't open statistics log file (-l)");
 			  exit(1);
 			}
+#if defined(F_SETFD)
+			fcntl(FILENO(statuslog), F_SETFD, 1);
+			/* close-on-exec */
+#endif
+			setvbuf(statuslog, (char *)NULL, _IOLBF, 0);
 			break;
 		case 'H':
 			if (hashlevels < 2)
@@ -403,10 +452,10 @@ main(argc, argv)
 	}
 	if (logfn != NULL) {
 	  /* loginit is a signal handler, so can't pass log */
-	  if (loginit(SIGHUP) < 0) /* do setlinebuf() there */
+	  if (loginitsched(SIGHUP) < 0) /* do setlinebuf() there */
 	    die(1, "log initialization failure");
 	  /* close and reopen log files */
-	  SIGNAL_HANDLE(SIGHUP, (RETSIGTYPE(*)())loginit);
+	  SIGNAL_HANDLE(SIGHUP, (RETSIGTYPE(*)())loginitsched);
 	} else {
 	  SIGNAL_IGNORE(SIGHUP); /* no surprises please */
 	  setvbuf(stdout, (char *)NULL, _IOLBF, 0);
