@@ -622,15 +622,15 @@ typedef	struct fd_set { fd_mask	fds_bits[1]; } fd_set;
 #endif	/* !NFDBITS */
 
 #ifdef FD_SET
-#define _FD_SET(sock,var) FD_SET(sock,&var)
-#define _FD_CLR(sock,var) FD_CLR(sock,&var)
-#define _FD_ZERO(var) FD_ZERO(&var)
-#define _FD_ISSET(i,var) FD_ISSET(i,&var)
+#define _Z_FD_SET(sock,var) FD_SET(sock,&var)
+#define _Z_FD_CLR(sock,var) FD_CLR(sock,&var)
+#define _Z_FD_ZERO(var) FD_ZERO(&var)
+#define _Z_FD_ISSET(i,var) FD_ISSET(i,&var)
 #else
-#define _FD_SET(sock,var) var |= (1 << sock)
-#define _FD_CLR(sock,var) var &= ~(1 << sock)
-#define _FD_ZERO(var) var = 0
-#define _FD_ISSET(i,var) ((var & (1 << i)) != 0)
+#define _Z_FD_SET(sock,var) var |= (1 << sock)
+#define _Z_FD_CLR(sock,var) var &= ~(1 << sock)
+#define _Z_FD_ZERO(var) var = 0
+#define _Z_FD_ISSET(i,var) ((var & (1 << i)) != 0)
 #endif
 
 int in_select = 0;
@@ -658,20 +658,20 @@ time_t timeout;
 	tv.tv_usec = 0;
 
 	maxf = 0;
-	_FD_ZERO(rdmask);
-	_FD_ZERO(wrmask);
+	_Z_FD_ZERO(rdmask);
+	_Z_FD_ZERO(wrmask);
 	readsockcnt = 0;
 	if (cpids != NULL)
 	  for (proc = cpids,i = 0; i < scheduler_nofiles ; ++i,++proc)
 	    if (proc->pid != 0) {
-	      _FD_SET(i, rdmask);
+	      _Z_FD_SET(i, rdmask);
 	      maxf = i;
 	      ++readsockcnt;
 	      if (proc->cmdlen != 0)
-		_FD_SET(i, wrmask);
+		_Z_FD_SET(i, wrmask);
 	    }
 	if (querysocket >= 0) {
-	  _FD_SET(querysocket, rdmask);
+	  _Z_FD_SET(querysocket, rdmask);
 	  if (maxf < querysocket)
 	    maxf = querysocket;
 	}
@@ -693,9 +693,9 @@ time_t timeout;
 	  if (errno == EINVAL || errno == EBADF) {
 	    fprintf(stderr, "** select() returned errno=%d\n", err);
 	    for (i = 0; i < maxf; ++i) {
-	      if (_FD_ISSET(i,rdmask)  &&  fcntl(i,F_GETFL,0) < 0)
+	      if (_Z_FD_ISSET(i,rdmask)  &&  fcntl(i,F_GETFL,0) < 0)
 		fprintf(stderr,"** Invalid fd on a select() rdmask: %d\n",i);
-	      if (_FD_ISSET(i,wrmask)  &&  fcntl(i,F_GETFL,0) < 0)
+	      if (_Z_FD_ISSET(i,wrmask)  &&  fcntl(i,F_GETFL,0) < 0)
 		fprintf(stderr,"** Invalid fd on a select() wrmask: %d\n",i);
 	    }
 	    fflush(stderr);
@@ -711,12 +711,12 @@ time_t timeout;
 	  return 1;
 	} else {
 	  /*fprintf(stderr, "got %d ready (%x)\n", n, rdmask.fds_bits[0]);*/
-	  if (querysocket >= 0 && _FD_ISSET(querysocket, rdmask)) {
+	  if (querysocket >= 0 && _Z_FD_ISSET(querysocket, rdmask)) {
 	    struct sockaddr_in raddr;
 	    int	raddrlen;
 
 	    --n;
-	    _FD_CLR(querysocket, rdmask);
+	    _Z_FD_CLR(querysocket, rdmask);
 	    raddrlen = sizeof raddr;
 	    i = accept(querysocket, (struct sockaddr *)&raddr, &raddrlen);
 	    if (i < 0) {
@@ -746,9 +746,9 @@ time_t timeout;
 	  }
 	  if (cpids != NULL) {
 	    for (i = 0; i < maxf; ++i) {
-	      if (cpids[i].pid != 0 && _FD_ISSET(i, rdmask)) {
+	      if (cpids[i].pid != 0 && _Z_FD_ISSET(i, rdmask)) {
 		--n;
-		_FD_CLR(i, rdmask);
+		_Z_FD_CLR(i, rdmask);
 		/*fprintf(stderr,"that is fd %d\n",i);*/
 		/* do non-blocking reads from this fd */
 		readfrom(i);
@@ -760,7 +760,7 @@ time_t timeout;
 	    for (i = 0; i < scheduler_nofiles; ++i)
 	      if (cpids[i].pid > 0
 		  && cpids[i].cmdlen != 0
-		  && _FD_ISSET(i, wrmask)) {
+		  && _Z_FD_ISSET(i, wrmask)) {
 		flush_child(&cpids[i]);
 		/* Because this loop might take a while ... */
 		queryipccheck();
@@ -783,12 +783,12 @@ queryipccheck()
 	  tv.tv_sec = 0;
 	  tv.tv_usec = 0;
 
-	  _FD_ZERO(mask);
-	  _FD_SET(querysocket, mask);
+	  _Z_FD_ZERO(mask);
+	  _Z_FD_SET(querysocket, mask);
 
 	  n = select(querysocket+1, &mask, NULL, NULL, &tv);
 	  if (n > 0 &&
-	      _FD_ISSET(querysocket, mask)) {
+	      _Z_FD_ISSET(querysocket, mask)) {
 	    struct sockaddr_in raddr;
 	    int raddrlen = sizeof(raddr);
 
@@ -796,6 +796,9 @@ queryipccheck()
 	    if (n >= 0) {
 	      int pid = fork();
 	      if (pid == 0) {
+#if defined(F_SETFD)
+		fcntl(n, F_SETFD, 1); /* close-on-exec */
+#endif
 #ifdef HAVE_TCPD_H /* TCP-Wrapper code */
 		if (wantconn(n, "mailq") == 0) {
 		  char *msg = "refusing 'mailq' query from your whereabouts\r\n";
@@ -849,6 +852,10 @@ queryipcinit()
 	  querysocket = -1;
 	  return;
 	}
+#if defined(F_SETFD)
+	fcntl(querysocket, F_SETFD, 1); /* close-on-exec */
+#endif
+
 	if (listen(querysocket, 5) < 0) {
 	  perror("listen:mailq socket");
 	  close(querysocket);
