@@ -680,9 +680,10 @@ static int u_ok(vp, index, inum, offset, notary, message)
 	char	*notary;
 	char	*message;
 {
+	if (verbose)
+	  fprintf(stderr,"%s: %d/%d/%s/ok %s\n", vp->cfp->logident, inum,
+		  offset, notary, message ? message : "-");
 #if 0
-	/* printf("%s: %d/%d/%s/ok %s\n", vp->cfp->logident, inum, offset,
-	   notary, message ? message : "-"); */
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
 	  fseek(vp->cfp->vfp, (off_t)0, SEEK_END);
 	  fprintf(vp->cfp->vfp, "%s: ok %s\n",
@@ -710,9 +711,13 @@ static int u_ok2(vp, index, inum, offset, notary, message)
 	char	*notary;
 	char	*message;
 {
+	if (vp->notaryflg & NOT_SUCCESS) {
+	  vp->cfp->haderror = 1; /* The transporter logged it for us! */
+	}
+	if (verbose)
+	  fprintf(stderr,"%s: %d/%d/%s/ok2 %s\n", vp->cfp->logident, inum,
+		  offset, notary, message ? message : "-");
 #if 0
-	/* printf("%s: %d/%d/%s/ok %s\n", vp->cfp->logident, inum, offset,
-	   notary, message ? message : "-"); */
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
 	  fseek(vp->cfp->vfp, (off_t)0, SEEK_END);
 	  fprintf(vp->cfp->vfp, "%s: ok %s\n",
@@ -720,10 +725,6 @@ static int u_ok2(vp, index, inum, offset, notary, message)
 		  message == NULL ? "(sent)" : message);
 	}
 #endif
-	if (vp->notaryflg & NOT_SUCCESS) {
-	  vp->cfp->haderror = 1; /* The transporter logged it for us! */
-	}
-
 	if (statuslog)
 	  logstat(statuslog,vp,"ok");
 
@@ -807,7 +808,7 @@ static int u_error2(vp, index, inum, offset, notary, message)
 	if (message == NULL)
 	  message = "(unknown)";
 	fprintf(stderr,
-		"%s %s: %ld/%ld/%s/error %s\n", timestring(),
+		"%s %s: %ld/%ld/%s/error2 %s\n", timestring(),
 		vp->thgrp->ce.command, inum, offset, notary, message);
 
 	/* We don't need to log it! */
