@@ -30,13 +30,15 @@ struct relaytest {
 #define ndbm  db_._ndbm
 };
 
-#else
+#else  /* This is the external interface -- doesn't tell a thing ;-) */
+
 struct relaytest {
   void *dummy;
 };
+
 #endif
 
 
 extern void reldefine __((struct relaytest **relp,char *dbtype, char *dbpath));
 extern int  reltest     __((struct relaytest *rel, char *str));
-extern int  reltestaddr __((struct relaytest *rel, struct sockaddr_in *raddr));
+extern int  reltestaddr __((struct relaytest *rel, struct sockaddr *raddr));
