@@ -60,6 +60,8 @@ char *Copyright2= "Copyright 1991-1997 Matti Aarnio";
 #endif
 #include "mail.h"
 
+#include <setjmp.h>
+
 #ifdef  HAVE_WAITPID
 # include <sys/wait.h>
 #else
@@ -196,6 +198,10 @@ int	pid, routerpid;
 FILE	*logfp = NULL;
 int	D_alloc = 0;
 
+jmp_buf jmpalarm;	/* Return-frame for breaking smtpserver
+			   when timeout hits.. */
+
+
 char *helplines[40] = { NULL, };
 
 static char *m200 = "2.0.0";
@@ -229,8 +235,10 @@ char	*style = "ve";
 struct smtpconf *cfinfo = NULL;
 long availspace = -1;	/* available diskspace/2 in bytes	*/
 long maxsize    = 0;
+int  use_ipv6   = 0;
+int  TcpRcvBufferSize = 0;
 int  percent_accept = 0;
-int maxloadavg = 999;	/* Maximum load-average that is tolerated
+int  maxloadavg = 999;	/* Maximum load-average that is tolerated
 			   with smtp-server actively receiving..
 			   Default value of 999 is high enough
 			   so that it will never block -- use
@@ -252,8 +260,19 @@ typedef struct {
 	int accept_sourcenet, accept_sourcedomain; /* =1, =1 */
 	int reject_net, reject_source;		   /* =0, =0 */
 	int rport;
-	char ihostaddr[3*4+1*3+2+1 /* == 18 for '[255.255.255.255]' */ +10/*slack*/];
-	struct sockaddr_in raddr;
+	char ihostaddr[sizeof("[ipv6.ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255]")+8];
+	union {
+	  struct sockaddr_in  v4;
+#ifdef AF_INET6
+	  struct sockaddr_in6 v6;
+#endif
+	} raddr;
+	union {
+	  struct sockaddr_in  v4;
+#ifdef AF_INET6
+	  struct sockaddr_in6 v6;
+#endif
+	} localsock;
 
 	int  s_bufread;
 	int  s_readout;
@@ -319,6 +338,9 @@ extern char **environ;
 extern int kill __((pid_t,int));
 static void reporterr __((SmtpState *, long, char*));
 
+extern char *rfc822atom __((char *str));
+extern char *xtext_string __((char *str));
+
 extern void s_setup __((SmtpState *SS, int fd, FILE *fp));
 extern int s_feof __((SmtpState *SS));
 extern int s_getc __((SmtpState *SS));
@@ -367,7 +389,7 @@ int insecure;
 	if (logfile != NULL) {
 		int fd;
 		if ((fd = open(logfile, O_CREAT|O_APPEND|O_WRONLY, 0644)) < 0) {
-			if ( !insecure )
+			if ( !insecure && daemon_flg)
 				fprintf(stderr,
 					"%s: cannot open logfile \"%s\": %s\n",
 					progname, logfile, strerror(errno));
@@ -387,9 +409,6 @@ main(argc, argv)
 {
 	int inetd, errflg, c, raddrlen, s, msgfd, version, i;
 	char *mailshare, path[1024];
-#ifdef	USE_INET
-	struct sockaddr_in sad;
-#endif	/* USE_INET */
 	u_short port = 0;
 	int port_set = 0;
 
@@ -419,8 +438,11 @@ main(argc, argv)
 	  exit(1);
 	}
 	/* optarg = NULL; */
-	while ((c = getopt(argc, argv, "aBd:ighl:np:L:M:P:R:s:Vv")) != EOF) {
+	while ((c = getopt(argc, argv, "6aBd:ighl:np:L:M:P:R:s:Vv")) != EOF) {
 	  switch (c) {
+	    case '6':
+		use_ipv6 = 1;
+		break;
 	    case 'a':
 		ident_flag = 1;
 		break;
@@ -541,12 +563,19 @@ main(argc, argv)
 	    }
 
 	    raddrlen = sizeof SS.raddr;
+	    memset(&SS.raddr, 0, raddrlen);
 	    if (getpeername(SS.inputfd,(struct sockaddr*)&SS.raddr,&raddrlen)){
 	      fprintf(stderr, "%s: getpeername(0): %s\n",
 		      progname, strerror(errno));
 	      exit(1);
 	    }
-	    SS.rport = SS.raddr.sin_port;
+#ifdef AF_INET6
+	    if (SS.raddr.v6.sin6_family == AF_INET6)
+	      SS.rport = SS.raddr.v6.sin6_port;
+	    else
+#endif
+	      SS.rport = SS.raddr.v4.sin_port;
+
 	    setrhostname(&SS);
 	    openlog("smtpserver", LOG_PID, LOG_MAIL);
 
@@ -591,7 +620,15 @@ main(argc, argv)
 	      fflush(stdout);
 	      fflush(stderr);
 	    }
-	    if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+#ifdef AF_INET6
+	    if (use_ipv6)
+	      s = socket(AF_INET6, SOCK_STREAM, 0 /* IPPROTO_IPV6 */);
+	    else
+	      s = socket(AF_INET,  SOCK_STREAM, 0 /* IPPROTO_IP   */);
+#else
+	    s = socket(AF_INET, SOCK_STREAM, 0);
+#endif
+	    if (s < 0) {
 	      fprintf(stderr,
 		      "%s: socket(AF_INET, SOCK_STREAM): %s\n",
 		      progname, strerror(errno));
@@ -612,6 +649,17 @@ main(argc, argv)
 	      exit(1);
 	    }
 #endif
+
+#ifdef SO_RCVBUF
+	    if (TcpRcvBufferSize > 0)
+	      if (setsockopt(s, SOL_SOCKET, SO_RCVBUF,
+			     (char *) &TcpRcvBufferSize,
+			     sizeof(TcpRcvBufferSize)) < 0) {
+		fprintf(stderr,"%s: setsockopt(SO_RCVBUF): %s\n",
+			progname, strerror(errno));
+		exit(1);
+	      }
+#endif
 	    if (port <= 0) {
 	      struct servent *service;
 #ifdef	IPPORT_SMTP
@@ -624,14 +672,42 @@ main(argc, argv)
 	      } else
 		port = service->s_port;
 	    }
-	    sad.sin_family = AF_INET;
-	    sad.sin_addr.s_addr = INADDR_ANY;
-	    sad.sin_port = port;
-	    if (bind(s, (struct sockaddr *)&sad, sizeof sad) < 0) {
-	      fprintf(stderr, "%s: bind(): %s\n",
-		      progname, strerror(errno));
-	      exit(1);
+
+#ifdef AF_INET6
+	    if (use_ipv6) {
+	      extern const struct in6_addr in6addr_any;
+
+	      struct sockaddr_in6 si6;
+	      memset(&si6,0,sizeof(si6));
+	      si6.sin6_family   = AF_INET6;
+	      si6.sin6_flowinfo = 0;
+	      si6.sin6_port     = port;
+	      si6.sin6_addr     = in6addr_any;
+
+	      i = bind(s, (struct sockaddr*)&si6, sizeof si6);
+	      if (i < 0) {
+		fprintf(stderr, "%s: bind(IPv6): %s\n",
+			progname, strerror(errno));
+		exit(1);
+	      }
 	    }
+	    else
+#endif
+	      {
+		struct sockaddr_in si4;
+
+		memset(&si4, 0, sizeof(si4));
+		si4.sin_family      = AF_INET;
+		si4.sin_addr.s_addr = INADDR_ANY;
+		si4.sin_port        = port;
+
+		i = bind(s, (struct sockaddr*)&si4, sizeof si4);
+		if (i < 0) {
+		  fprintf(stderr, "%s: bind(IPv4): %s\n",
+			  progname, strerror(errno));
+		  exit(1);
+		}
+	      }
 
 	    /* Set the listen limit HIGH; there has been an active
 	       denial-of-service attack where people send faked SYNs
@@ -746,7 +822,13 @@ main(argc, argv)
 
 		SIGNAL_HANDLE(SIGTERM, SIG_IGN);
 
-		SS.rport = SS.raddr.sin_port;
+#ifdef AF_INET6
+		if (SS.raddr.v6.sin6_family == AF_INET6)
+		  SS.rport = SS.raddr.v6.sin6_port;
+		else
+#endif
+		  SS.rport = SS.raddr.v4.sin_port;
+
 		setrhostname(&SS);
 
 		openlog("smtpserver", LOG_PID, LOG_MAIL);
@@ -845,11 +927,33 @@ setrhostname(SS)
 SmtpState *SS;
 {
 	struct hostent *hp;
+	if (SS->raddr.v4.sin_family == AF_INET)
+	  inet_ntop(AF_INET, &SS->raddr.v4.sin_addr,  /* IPv4 */
+		    SS->ihostaddr+1, sizeof(SS->ihostaddr)-2);
+#ifdef AF_INET6
+	else if (SS->raddr.v6.sin6_family == AF_INET6) {
+	  strcpy(SS->ihostaddr+1,"ipv6.");
+	  inet_ntop(AF_INET6, &SS->raddr.v6.sin6_addr, /* IPv6 */
+		    SS->ihostaddr+6, sizeof(SS->ihostaddr)-7);
+	}
+#endif
+	else {
+	  ; /* XX: ??? Not AF_INET, nor AF_INET6 ??? */
+	}
+	SS->ihostaddr[0] = '[';
+	strcat(SS->ihostaddr,"]");
 
- 	sprintf(SS->ihostaddr, "[%s]", dottedquad(&SS->raddr.sin_addr));
 	if (skeptical) {
-	  hp = gethostbyaddr((char *)&(SS->raddr.sin_addr),
-			     sizeof (struct in_addr), SS->raddr.sin_family);
+	  if (SS->raddr.v4.sin_family == AF_INET)
+	    hp = gethostbyaddr((char*)&SS->raddr.v4.sin_addr,   4, AF_INET);
+#ifdef AF_INET6
+	  else if (SS->raddr.v6.sin6_family == AF_INET6)
+	    hp = gethostbyaddr((char*)&SS->raddr.v6.sin6_addr, 16, AF_INET6);
+#endif
+	  else {
+	    ; /* XX: ??? Not AF_INET, nor AF_INET6 ??? */
+	  }
+
 	  if (hp != NULL)
 	    strcpy(SS->rhostname, hp->h_name);
 	  else
@@ -864,17 +968,11 @@ static RETSIGTYPE
 timedout(sig)
 int sig;
 {
-SmtpState *SS = NULL;
-	reporterr(SS,0,"SMTP protocol timed out");
-
-	/* If there is something going on, kill the file.. */
-	if (SS->mfp != NULL)
-	  mail_abort(SS->mfp);
-	SS->mfp = NULL;
-
-	if (routerpid > 0)
-	  killr(SS, routerpid);
-	exit(0);
+	/* Return to the smtpserver's mail-program.
+	   We are commiting a suicide, but we need
+	   data that exists only in that context... */
+	longjmp(jmpalarm, 1);
+	_exit(253); /* We did return ??!?! Boo!!! */
 }
 
 static RETSIGTYPE
@@ -907,7 +1005,7 @@ reporterr(SS, tell, msg)
 	syslog(LOG_ERR,
 	       "aborted (%ld bytes) from %s/%d: %s", tell, SS->rhostname, SS->rport, msg);
 	if (logfp != NULL) {
-	  fprintf(logfp, "%d#\taborted: %s\n", pid, msg);
+	  fprintf(logfp, "%d#\taborted (%ld bytes): %s\n", pid, tell, msg);
 	  fflush(logfp);
 	}
 }
@@ -937,7 +1035,7 @@ int s_getc(SS)
 	  SS->s_readout = 0;
 	}
 	if (rc) return EOF;
-	return (unsigned char) SS->s_buffer[SS->s_readout++];
+	return ((unsigned char*) SS->s_buffer)[SS->s_readout++];
 }
 
 int s_hasinput(SS)
@@ -967,92 +1065,31 @@ void s_setup(SS, infd, outfp)
 	int infd;
 	FILE *outfp;
 {
-	SS->inputfd = infd;
+	SS->inputfd   = infd;
 	SS->outfp     = outfp;
-	SS->s_status = 0;
+	SS->s_status  =  0;
 	SS->s_bufread = -1;
-	SS->s_readout = 0;
+	SS->s_readout =  0;
 }
 
 
 
-
-extern char *xtext_string __((char *str));
-char *xtext_string(str)
-char *str;
-{
-	/* Verify that the input is valid RFC 1981 XTEXT string! */
-	char *str0 = str;
-
-	while (*str) {
-	  unsigned char c = *str;
-
-	  if (c == ' ' || c == '\t')
-	    break;
-
-	  if ('!' <= c && c <= '~' && c != '+' && c != '=')
-	    ; /* is ok! */
-	  else if (c == '+') {
-	    c = *++str;
-	    if (!(('0' <= c && c <= '9') || ('A' <= c && c <= 'F'))) {
-	      goto err;
-	    }
-	    c = *++str;
-	    if (!(('0' <= c && c <= '9') || ('A' <= c && c <= 'F'))) {
-	      goto err;
-	    }
-	  } else {
-	    goto err;
-	  }
-	  ++str;
-	}
-	if (str == str0) {
-	err:
-	  rfc821_error_ptr = str;
-	  return NULL;
-	}
-	return str;
-}
-
-char *rfc822atom(str)
-char *str;
-{
-	char *str0 = str;
-	while (*str != 0) {
-	  u_char c = *str;
-	  /* XX: SHOULD do this with a charmap test! */
-	  if (c <= ' ' || c > 126 ||
-	      c == '(' || c == ')' ||
-	      c == '<' || c == '>' ||
-	      c == ',' || c == ';' ||
-	      c == ':' || c == '\\' ||
-	      c == '"' || c == '[' || c == ']') {
-	    if (str > str0)
-	      return str;
-	    goto err;
-	  }
-	  ++str;
-	}
-	if (str == str0) {
-	err:
-	  rfc821_error_ptr = str;
-	  return NULL;
-	}
-	return str;
-}
-
 char *orcpt_string(str)
 char *str;
 {
 	/* Verify that the input is valid RFC 1981 XTEXT string! */
-	str = rfc822atom(str);
+	char *stro = rfc822atom(str);
+	char *s2;
 
-	if (str == NULL) return NULL;
-	if (*str != ';') {
-	  rfc821_error_ptr = str-1;
+	if (stro == str) return NULL;
+	if (*stro != ';') {
+	  rfc821_error_ptr = stro-1;
 	  return NULL;
 	}
-	return xtext_string(++str);
+	s2 = xtext_string(++stro);
+	if (s2 == stro)
+	  return NULL;
+	return s2;
 }
 
 
@@ -1065,7 +1102,7 @@ char *str;
 #define RFC821_822QUOTE(cp) \
 	if (cp && strchr(cp,'\\') != NULL && *cp != '"') {	\
 	  char *s1 = cp, *s2;					\
-	  s2 = newcp = alloca(strlen(cp)*2);			\
+	  s2 = newcp = alloca(strlen(cp)+8);			\
 	  *s2++ = '"';						\
 	  while (*s1) {						\
 	    if (*s1 == '@')					\
@@ -1084,7 +1121,7 @@ char *str;
 #define RFC821_822QUOTE(cp) \
 	if (cp && strchr(cp,'\\') != NULL && *cp != '"') {	\
 	  char *s1 = cp, *s2;					\
-	  s2 = newcp = emalloc(strlen(cp)*2);			\
+	  s2 = newcp = emalloc(strlen(cp)+8);			\
 	  *s2++ = '"';						\
 	  while (*s1) {						\
 	    if (*s1 == '@')					\
@@ -1287,7 +1324,7 @@ int insecure;
 	s = rfc821_path(cp,STYLE(cfinfo, 'R'));
 	if (s == cp) {
 	  /* Failure.. */
-	  type821err(501,m552,buf,"Path data: %.200s",rfc821_error);
+	  type821err(SS,501,m552,buf,"Path data: %.200s",rfc821_error);
 	  return;
 	}
 	if (*s == '>') {
@@ -1377,8 +1414,8 @@ int insecure;
 	    }
 	    drpt_envid = s+6;
 	    s = xtext_string(s+6);
-	    if (s == NULL) {
-	      type821err(-501,m554,buf,"Invalid ENVID value '%.200s'", drpt_envid);
+	    if (s == s+6) {
+	      type821err(SS,-501,m554,buf,"Invalid ENVID value '%.200s'", drpt_envid);
 	      type(SS,501,m554,"ENVID data contains illegal characters!");
 	      rc = 1;
 	      break;
@@ -1548,7 +1585,7 @@ char *buf, *cp;
 	s = rfc821_path(cp,STYLE(cfinfo, 'R'));
 	if (s == cp) {
 	  /* Failure.. */
-	  type821err(501,m552,buf,"Path data: %s",rfc821_error);
+	  type821err(SS,501,m552,buf,"Path data: %s",rfc821_error);
 	  return;
 	}
 	if (*s == '>') {
@@ -1606,7 +1643,7 @@ char *buf, *cp;
 	    drpt_orcpt = s;
 	    s = orcpt_string(s+6);
 	    if (s == NULL) {
-	      type821err(-501,m454,buf,"Invalid ORCPT value '%s'", drpt_orcpt);
+	      type821err(SS,-501,m454,buf,"Invalid ORCPT value '%s'", drpt_orcpt);
 	      type(SS,501,m454,"ORCPT-param data error!");
 	      return;
 	    }
@@ -1980,12 +2017,13 @@ smtp_verify(SS,buf,cp,cfinfo)
 
 	cfi = STYLE(cfinfo, 'v');
 
+#if 0 /* The input string is ARBITARY STRING (shudder..) */
 	if (*cp == '<')
 	  s = rfc821_path(cp,cfi); /* with < > */
 	else
 	  s = rfc821_path2(cp,cfi); /* Without < > */
 	if (s == cp) {
-	  type821err(501,m552, buf,"Path data: %s",rfc821_error);
+	  type821err(SS,501,m552, buf,"Path data: %s",rfc821_error);
 	  return;
 	}
 	while (*s == ' ' || *s == '\t') ++s;
@@ -1993,6 +2031,7 @@ smtp_verify(SS,buf,cp,cfinfo)
 	  type(SS,501, m552, "Growl! Extra junk after the VRFY argument!");
 	  return;
 	}
+#endif
 
 	RFC821_822QUOTE(cp) ;
 
@@ -2026,6 +2065,7 @@ smtp_expand(SS,buf,cp,cfinfo)
 	}
 	if ((cfi = STYLE(cfinfo, 'e'))) {
 	  while (*cp == ' ' || *cp == '\t') ++cp;
+#if 0  /* The input string is an arbitary string! */
 	  if (*cp == '<')
 	    s = rfc821_path(cp,cfi); /* with < > */
 	  else
@@ -2039,6 +2079,8 @@ smtp_expand(SS,buf,cp,cfinfo)
 	    type(SS,501, m552, "Growl! Extra junk after the EXPN argument!");
 	    return;
 	  }
+#else
+#endif
 
 	  RFC821_822QUOTE(cp) ;
 
@@ -2063,8 +2105,9 @@ smtpserver(SS,insecure)
 {
 	char *cp;
 	time_t now;
-	struct sockaddr_in localsock;
 	int localsocksize;
+	int rc;
+	long tell;
 
 	int VerboseCommand = 0;
 
@@ -2074,6 +2117,29 @@ smtpserver(SS,insecure)
 
 	runastrusteduser();
 
+	rc = setjmp(jmpalarm);
+	if (rc != 0) {
+	  /* Oooo...  We are returning here via  longjmp(),
+	     which means we just got a timeout (SIGALRM),
+	     which for us is instant death.. */
+	  tell = 0;
+	  if (SS->mfp != NULL) {
+	    fseek(SS->mfp, 0, SEEK_END);
+	    tell = ftell(SS->mfp);
+	  }
+	  reporterr(SS, tell, "SMTP protocol timed out");
+
+	  /* If there is something going on, kill the file.. */
+	  if (SS->mfp != NULL)
+	    mail_abort(SS->mfp);
+	  SS->mfp = NULL;
+
+	  if (routerpid > 0)
+	    killr(SS, routerpid);
+	  exit(0);
+	}
+
+
 	report(SS,"(connected)");
 	now = time((time_t *)0);
 	cp = (char *)rfc822date(&now);
@@ -2083,24 +2149,29 @@ smtpserver(SS,insecure)
 	/* Lets figure-out who we are this time around -- we may be on
 	   a machine with multiple identities per multiple interfaces,
 	   or via virtual IP-numbers, or ... */
-	localsocksize = sizeof(localsock);
-	if (getsockname(SS->inputfd,
-			(struct sockaddr *)&localsock,&localsocksize) == 0 &&
-	    localsocksize == sizeof(localsock)) {
+	localsocksize = sizeof(SS->localsock);
+	if (getsockname(SS->inputfd, (struct sockaddr*)&SS->localsock,
+			&localsocksize) == 0) {
 	  /* Success with it :) */
-	  struct hostent *hostent = gethostbyaddr((void*)&localsock.sin_addr,4,
-						  localsock.sin_family);
+	  struct hostent *hostent;
+#ifdef AF_INET6
+	  if (SS->localsock.v6.sin6_family == AF_INET6)
+	    hostent = gethostbyaddr((void*)&SS->localsock.v6.sin6_addr, 16, AF_INET6);
+	  else
+#endif
+	    hostent = gethostbyaddr((void*)&SS->localsock.v4.sin_addr,   4, AF_INET );
+
 	  if (hostent)
-	    strcpy(SS->myhostname,hostent->h_name);
+	    strcpy(SS->myhostname, hostent->h_name);
 	}
 
 	SS->reject_net = 0; /* Default for source net rejection */
 	if (rejectnets != NULL)
-	  SS->reject_net = reltestaddr(rejectnets,&SS->raddr);
+	  SS->reject_net = reltestaddr(rejectnets, (void*)&SS->raddr);
 	SS->accept_sourcenet = 1; /* Default */
 	if (relaytargets != NULL &&
 	    relaysourcenets != NULL) {
-	  SS->accept_sourcenet = reltestaddr(relaysourcenets,&SS->raddr);
+	  SS->accept_sourcenet = reltestaddr(relaysourcenets, (void*)&SS->raddr);
 	}
 
 	/* re-opening the log ?? */
@@ -2110,7 +2181,7 @@ smtpserver(SS,insecure)
 	if (wantconn(SS->inputfd, "smtp-receiver") == 0) {
 	  syslog(LOG_WARNING, "refusing connection from %s/%s",
 		 SS->rhostname, SS->ident_username);
-	  type(SS,421, NULL, "%s ZMailer Server %s WILL NOT TALK TO YOU at %s",
+	  type(SS,421,NULL, "%s ZMailer Server %s WILL NOT TALK TO YOU at %s",
 	       SS->myhostname, VersionNumb, cp);
 	  typeflush(SS);
 	  sleep(1);
@@ -2206,7 +2277,7 @@ smtpserver(SS,insecure)
 
 	  if (rc >= 0) {
 	    rfc821_error_ptr = buf + rc;
-	    type821err(500, m552, buf,
+	    type821err(SS,500, m552, buf,
 		       "Illegal input characters: %s",
 		       ((buf[rc]=='\0') ? "NUL on SMTP input" :
 			((buf[rc]&0x80) ? "8-bit char on SMTP input":
@@ -2362,7 +2433,8 @@ smtpserver(SS,insecure)
 	if (SS->mfp != NULL)
 	  mail_abort(SS->mfp);
 	SS->mfp = NULL;
-	reporterr(SS,lseek(0,(off_t)0,SEEK_CUR), "session terminated");
+	tell = lseek(0,0,SEEK_CUR);
+	reporterr(SS, tell, "session terminated");
 }
 
 #define	TYPE_(m)	type(SS,-214, NULL, "%s", m);
@@ -3165,6 +3237,8 @@ mvdata(SS, msg)
 	endstate = X_;
 	sts = states;
 	cnt = 0;
+
+	alarm(SMTP_DATA_TIME_PER_LINE);
 #else
 	char linebuf[4000], *s, *eol;
 	int col;
@@ -3363,10 +3437,8 @@ mvbdata(SS, msg, incount)
 	/* ================ Normal email BODY input.. ================ */
 	for ( ; incount > 0; --incount) {
 		c = s_getc(SS);
-#if EOF != -1
 		if (c == EOF)		/* a little slower... */
 			break;
-#endif
 		if (--timercnt <= 0) {
 			/* Re-arm for every kilobyte */
 			alarm(SMTP_DATA_TIME_PER_LINE);
@@ -3414,20 +3486,32 @@ SmtpState *SS;
     int rc;
     char identbuf[1024];
 
-    if ((rc = ident_fd2(SS->inputfd, &ident_inlocal, &ident_inremote,
-			&ident_local, &ident_remote)) != 0) {
-      sprintf(SS->ident_username, "IDENT-FD2-FAULT-%d-%d", rc, errno);
-      return;
-    } else {
-      cp = ident_tcpuser4(&ident_inlocal, &ident_inremote,
-			  ident_local,   ident_remote,
+#ifdef AF_INET6
+    if (SS->raddr.v6.sin6_family == AF_INET6) {
+      cp = ident_tcpuser9(SS->raddr.v6.sin6_family, 16,
+			  &SS->localsock.v6.sin6_addr,
+			  &SS->raddr.v6.sin6_addr,
+			  ntohs(SS->localsock.v6.sin6_port),
+			  ntohs(SS->raddr.v6.sin6_port),
 			  IDENT_TIMEOUT,
 			  identbuf, sizeof(identbuf)-1);
-      if (cp != NULL)
-	strncpy(SS->ident_username, cp, MAXHOSTNAMELEN);
-      else
-	strncpy(SS->ident_username, "IDENT-CALL-FAULT", MAXHOSTNAMELEN);
-    }
+    } else
+#endif
+      if (SS->raddr.v4.sin_family == AF_INET)
+	cp = ident_tcpuser9(SS->raddr.v4.sin_family, 4,
+			    &SS->localsock.v4.sin_addr,
+			    &SS->raddr.v4.sin_addr,
+			    ntohs(SS->localsock.v4.sin_port),
+			    ntohs(SS->raddr.v4.sin_port),
+			    IDENT_TIMEOUT,
+			    identbuf, sizeof(identbuf)-1);
+      else {
+	cp = "Unknown_type_of_remote_system_address!";
+      }
+    if (cp != NULL)
+      strncpy(SS->ident_username, cp, MAXHOSTNAMELEN);
+    else
+      strncpy(SS->ident_username, "IDENT-CALL-FAULT", MAXHOSTNAMELEN);
 }
 
 
@@ -3503,6 +3587,9 @@ char *str;
     reldefine(&rejecttarget,param1,param2);
     return;
   }
+  if (strcasecmp(name,"tcprcvbuffersize") == 0) {
+    sscanf(param1,"%d",&TcpRcvBufferSize);
+  }
 }
 
 struct smtpconf *
