@@ -1,5 +1,5 @@
 #! /bin/sh
-realprog=/opt/mail/bin/vacation
+realprog=/opt/mail/bin/vacation.exe
 case "$HOME" in
 '')	echo "$0: HOME environment variable not set. Something is wrong." >&2
 	exit 1;;
@@ -40,7 +40,7 @@ start)
 stop)
 	if test ! -f $HOME/.forward; then
 		echo "$0: No $HOME/.forward file. Something is wrong." >&2
-		exit 1;
+		exit 1
 	fi
 	rm $HOME/.forward
 	;;
@@ -49,5 +49,6 @@ stop)
 	echo "$0: Wrong usage!"
 	echo "  $0 $@"
 	echo "Parameter is either: start  or  stop!"
+	exit 64
 	;;
 esac
