@@ -13,6 +13,7 @@
 #include <sys/param.h>
 #include <sys/stat.h>
 #include "mail.h"
+#include "syslog.h"
 #include "zmsignal.h"
 #include "ta.h"
 
@@ -104,6 +105,9 @@ main(argc, argv)
 	  exit(EX_USAGE);
 	}
 
+	/* We need this latter on .. */
+	openlog("errormail", LOG_PID, LOG_MAIL);
+
 	while (!getout) {
 	  char *s, *host;
 
