@@ -64,6 +64,9 @@ typedef char msgdata;
 #ifndef T_NSAP
 # define T_NSAP 22
 #endif
+#ifndef T_AAAA
+# define T_AAAA 28	/* IPv6 Address */
+#endif
 #ifndef T_NSAP_PTR
 # define T_NSAP_PTR 23
 #endif
@@ -443,6 +446,7 @@ struct qtypes {
 	{	"cname",	T_CNAME		},
 	{	"mx",		T_MX		},
 	{	"a",		T_A		},
+	{	"aaaa",		T_AAAA		},
 #ifdef  T_ANY
 	{	"any",		T_ANY		},
 #endif
@@ -466,6 +470,7 @@ hold_ns(s)
 {
 	struct qtypes *qtp;
 	char *cp, host[1024]; /* 256 should be enough .. */
+	int ttl;
 
 	res_init();
 
@@ -488,7 +493,7 @@ hold_ns(s)
 	}
 	strncpy(host, s, sizeof(host));
 	host[sizeof(host)-1] = 0;
-	switch (getrrtype(host, sizeof host, qtp->value, 2, NULL)) {
+	switch (getrrtype(host, &ttl, sizeof host, qtp->value, 2, NULL)) {
 	case 0:
 		return 1;	/* negative reply */
 	case 1:
@@ -545,6 +550,7 @@ hold_ns(s)
 	  hp = gethostbyname(host);
 	  break;
 	case 2:
+/* XXX: use  inet_pton() here ?! */
 	  ia.s_addr = inet_addr(host);
 	  hp = gethostbyaddr(&ia, sizeof ia.s_addr, AF_INET);
 	  break;
