@@ -30,7 +30,9 @@ extern void *erealloc();
 
 int cvtspace_copy __((struct rcpt *rp));
 
-#define USE_NEW_MIME2_CODE /* TESTING! */ /* not yet ready.. */
+#if 0
+#define USE_NEW_MIME2_CODE /* TESTING! */ /* apparenly not yet ready.. */
+#endif
 #ifdef USE_NEW_MIME2_CODE
 
 /*
@@ -73,7 +75,8 @@ header8bit2QPnextword(line, word, len, inside, start_encode, fold, hdr_offset)
 	    eightbitchars += 2;
 	  wordlen++;
 	}
-	if (wordlen+(*len)+has8bit*(eightbitchars) >= MAXENCODEDLINE) {
+	if (wordlen &&
+	    wordlen+(*len)+has8bit*(eightbitchars) >= MAXENCODEDLINE) {
 	  has8bit = 1;
 	  wordlen--;
 	  c = *(*line+wordlen);
@@ -167,7 +170,7 @@ header8bit2QP(verboselog, line, defcharset, outp, sizep, osizep, fold)
 	  hdr_offset++;
 	ptr = line;
 #endif
-	while (*ptr != '\0') {
+	while (*ptr != '\0' && lines == 0) {
 	  int wlen;
 	  if (header8bit2QPnextword(&ptr, word, &len, &inside, tmp, fold, hdr_offset)) {
 	    lines++;
