@@ -154,8 +154,15 @@ struct rcpt *rp;
 }
 
 #ifdef HAVE_STDARG_H
+#ifdef __STDC__
 int
 append_header(struct rcpt *rp, char *fmt, ...)
+#else /* Not ANSI-C */
+int
+append_header(rp, fmt)
+	struct rcpt *rp;
+	char *fmt;
+#endif
 #else
 int
 append_header(va_alist)
