@@ -61,6 +61,17 @@ char *msg;
      ctladdr=`getpwuid(rp->addr->misc)`
      mailer='rp->addr->channel' */
 
+  if (wtthost != NULL) {
+    char *s = strchr(wtthost,';');
+    if (s) ++s;
+    while (s && (*s == ' ')) ++s;
+    if (s)
+      wtthost = s;
+  }
+
+  if (strcmp(stat,"ok")==0 || strcmp(stat,"ok2")==0)
+    msg = ""; /* Shut up ... */
+
   if (wtthost == NULL)
     sprintf(linebuf, "%s: to=%.200s, delay=%s, xdelay=%s, mailer=%.200s, stat=%.200s %.400s",
 	    spoolid, rp->addr->user, delays, xdelays, rp->addr->channel, stat, msg);
