@@ -49,7 +49,7 @@
 #include "zmsignal.h"
 #include "ta.h"
 #include "splay.h"
-#include "malloc.h"
+/* #include "malloc.h" */
 
 #ifdef  HAVE_WAITPID
 # include <sys/wait.h>
@@ -245,6 +245,16 @@ struct userhost {
 	char		*hostname;
 };
 
+struct writestate {
+	FILE *fp;
+	int  lastch;
+	char expect;
+	char frombuf[8];
+	char *fromp;
+	char *buf2; /* writemimeline() processing aux buffer */
+	int buf2len;
+};
+
 struct sptree *spt_users = NULL;
 #endif /* HAVE_SOCKET -- no use to biff, if no socket available .. */
 
@@ -282,8 +292,10 @@ extern int emptyline();
 extern void biff __((char *, char *, long));
 extern void rbiff __((struct biffer *));
 extern void prversion __((const char *)), getnobody();
-extern struct passwd *getpwnam();
-extern struct passwd *getpwuid();
+#if 0
+extern struct passwd *getpwnam __((const char *));
+extern struct passwd *getpwuid __((uid_t));
+#endif
 extern int setupuidgid __((struct rcpt *, int, int));
 extern int createfile __((struct rcpt *, char *, uid_t));
 extern int exstat __((struct rcpt *, char *, struct stat *, int (*)() ));
@@ -296,8 +308,8 @@ extern void program __((struct ctldesc *dp, struct rcpt *rp, char *timestring));
 extern FILE *putmail __((struct ctldesc *dp, struct rcpt *rp, int fdmail, char *fdopmode, char *timestring, char *file));
 extern int appendlet __((struct ctldesc *dp, struct rcpt *rp, FILE *fp, char *file, int ismime));
 extern char **environ;
-extern int writebuf __((FILE *fp, char *buf, int len, int *lastchp));
-extern int writemimeline __((FILE *fp, char *buf, int len, int *lastchp));
+extern int writebuf __((struct writestate *, char *buf, int len));
+extern int writemimeline __((struct writestate *, char *buf, int len));
 
 static int do_return_receipt_to = 0;
 static void  return_receipt __((struct ctldesc *dp, char *retrecpaddr, char *uidstr));
@@ -306,6 +318,8 @@ static char *find_return_receipt_hdr __((struct rcpt *rp));
 extern int  qp_to_8bit   __((struct rcpt *rp));
 extern int  qptext_check __((struct rcpt *rp));
 
+extern char *rfc822date __((time_t *));
+
 extern void hp_init();
 extern char **hp_getaddr();
 extern int errno;
@@ -315,7 +329,7 @@ extern char *strchr(), *strrchr();
 extern int  lstat(); /* remaps to  stat() if USE_LSTAT is not defined.. */
 extern FILE *fdopen();
 #ifndef MALLOC_TRACE
-extern univptr_t emalloc();
+extern void * emalloc __((size_t));
 #else
 struct conshell *envarlist = NULL;
 #endif
@@ -577,6 +591,7 @@ process(dp)
 	 * and strip the quotes from around a name.
 	 */
 	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
+	  int isquoted = 0;
 	  while (*(rp->addr->user) == TO_USER)
 	    rp->addr->user++;
 	  if ((at   = strrchr(rp->addr->user,'@')) != NULL) *at = 0;
@@ -586,9 +601,11 @@ process(dp)
 	    if (*cp == QUOTE) {
 	      *cp = '\0';
 	      rp->addr->user++;
+	      isquoted = 1;
 	    }
 	  }
-	  if (at) *at = '@';	/* If it followed quotes, a null prefixes it.. */
+	  if (at) *at = '@';	/* If it followed quotes,
+				   a null prefixes it.. */
 	  if (plus) *plus = '+';
 #ifdef  HAVE_SOCKET
 #ifdef	HAVE_PROTOCOLS_RWHOD_H /* RBIFF */
@@ -890,7 +907,7 @@ deliver(dp, rp, timestring)
 	    }
 	    return;
 	  }
-	  if (!setupuidgid(rp, uid, -1)) {
+	  if (!setupuidgid(rp, uid, 1)) {
 	    setrootuid(rp);
 	    return;
 	  }
@@ -1883,10 +1900,11 @@ setupuidgid(rp, uid, gid)
 	int gid;
 {
 	setrootuid(rp);
-	if (setgid(gid) < 0) {
-	  DIAGNOSTIC(rp, EX_OSERR, "can't setgid to %d", (int)gid);
-	  return 0;
-	}
+	if (gid >= 0)
+	  if (setgid(gid) < 0) {
+	    DIAGNOSTIC(rp, EX_OSERR, "can't setgid to %d", (int)gid);
+	    return 0;
+	  }
 	if (*(rp->addr->user) == TO_FILE || *(rp->addr->user) == TO_PIPE)
 	  uid = atol(rp->addr->misc);
 	if (setreuid(-1, uid) < 0) {
@@ -1966,21 +1984,27 @@ appendlet(dp, rp, fp, file, ismime)
 	int ismime;
 	char *file;
 {
-	int lastch = 0xFFF; /* Something no character can be,
-			       and not "-1" either.. */
+	struct writestate WS;
+	char *s; /* message body start */
+
 #ifndef	HAVE_MMAP
-	static char buffer[BUFSIZ+1];
-	static char *bufend = buffer+BUFSIZ;
+	char buffer[BUFSIZ+1];
+	char *bufend = buffer+BUFSIZ;
 	register int i;
 	register int bufferfull;
-	char iobuf[BUFSIZ];
+	char iobuf[BUFSIZ]; /* For setvbuf(mfp) */
 	FILE *mfp;
 	int mfd = dp->msgfd;
+#endif
 
-	if (ismime)
-	  writemimeline(NULL, NULL, 0, NULL);	/* initialize state */
-	else
-	  writebuf(NULL, NULL, 0, NULL);	/* initialize state */
+	WS.fp     = fp;
+	WS.expect = mmdf_mode ? 0 : 'F';
+	WS.lastch = 256;	/* Something no character can be,
+				   and not "-1" either.. */
+	WS.frombuf[0] = 0;
+	WS.fromp = WS.frombuf;
+
+#ifndef HAVE_MMAP
 
 	if (ismime) {
 	  /* can we use cache of message body data ? */
@@ -1997,7 +2021,7 @@ appendlet(dp, rp, fp, file, ismime)
 	      if (*s == '\n' && readidx < readalready) {
 		++s; ++linelen; ++readidx;
 	      }
-	      if (writemimeline(fp, s0, linelen, &lastch) != linelen) {
+	      if (writemimeline(&WS, s0, linelen) != linelen) {
 		DIAGNOSTIC(rp, EX_IOERR,
 			   "write to \"%s\" failed", file);
 		return -256;
@@ -2005,7 +2029,7 @@ appendlet(dp, rp, fp, file, ismime)
 	      s0 = s;
 	    }
 	    rp->status = EX_OK;
-	    return lastch;
+	    return WS.lastch;
 	  }
 
 	  mfp = fdopen(mfd,"r");
@@ -2024,16 +2048,14 @@ appendlet(dp, rp, fp, file, ismime)
 	    if (i == BUFSIZ && buffer[BUFSIZ-1] != '\n')
 	      ismime = 0;
 	    /* Ok, write the line */
-	    if (writemimeline(fp, buffer, i, &lastch) != i) {
-	      DIAGNOSTIC(rp, EX_IOERR,
-			 "write to \"%s\" failed", file);
+	    if (writemimeline(&WS, buffer, i) != i) {
+	      DIAGNOSTIC(rp, EX_IOERR, "write to \"%s\" failed", file);
  	      MFPCLOSE;
 	      return -256;
 	    }
 	  }
 	  if (i == EOF && !feof(mfp)) {
-	    DIAGNOSTIC(rp, EX_IOERR,
-		       "read error from message file", 0);
+	    DIAGNOSTIC(rp, EX_IOERR, "read error from message file", 0);
 	    MFPCLOSE;
 	    return -256;
 	  }
@@ -2044,13 +2066,13 @@ appendlet(dp, rp, fp, file, ismime)
 
 	  /* can we use cache of message body data ? */
 	  if (readalready != 0) {
-	    if (writebuf(fp, buffer, readalready, &lastch) != readalready) {
+	    if (writebuf(&WS, buffer, readalready) != readalready) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
 	      return -256;
 	    }
 	    rp->status = EX_OK;
-	    return lastch;
+	    return WS.lastch;
 	  }
 
 	  /* we are assumed to be positioned properly
@@ -2058,13 +2080,11 @@ appendlet(dp, rp, fp, file, ismime)
 	  bufferfull = 0;
 	  while ((i = read(mfd, buffer, BUFSIZ)) != 0) {
 	    if (i < 0) {
-	      DIAGNOSTIC(rp, EX_IOERR,
-			 "read error from message file", 0);
+	      DIAGNOSTIC(rp, EX_IOERR, "read error from message file", 0);
 	      return -256;
 	    }
-	    if (writebuf(fp, buffer, i, &lastch) != i) {
-	      DIAGNOSTIC(rp, EX_IOERR,
-			 "write to \"%s\" failed", file);
+	    if (writebuf(&WS, buffer, i) != i) {
+	      DIAGNOSTIC(rp, EX_IOERR, "write to \"%s\" failed", file);
 	      return -256;
 	    }
 	    readalready = i;
@@ -2075,17 +2095,16 @@ appendlet(dp, rp, fp, file, ismime)
 	    readalready = 0;
 	}
 #else /* HAVE_MMAP  --  get the input from  mmap():ed memory area.. */
-	char *s; /* message body start */
 	s = dp->let_buffer + dp->msgbodyoffset;
 	if (ismime) {
 	  int i;
-	  writemimeline(NULL, NULL, 0, NULL);	/* initialize state */
+
 	  while (s < dp->let_end) {
 	    char *s2 = s;
 	    i = 0;
 	    while (s2 < dp->let_end && *s2 != '\n') ++s2, ++i;
 	    if (s2 < dp->let_end && *s2 == '\n') ++s2, ++i;
-	    if (writemimeline(fp, s, i, &lastch) != i) {
+	    if (writemimeline(&WS, s, i) != i) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
 	      return -256;
@@ -2093,15 +2112,14 @@ appendlet(dp, rp, fp, file, ismime)
 	    s = s2;
 	  }
 	} else {
-	  writebuf(NULL, NULL, 0, NULL);	/* initialize state */
-	  if (writebuf(fp, s, dp->let_end - s, &lastch) != (dp->let_end - s)) {
+	  if (writebuf(&WS, s, dp->let_end - s) != (dp->let_end - s)) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
 	      return -256;
 	  }
 	}
 #endif
-	return lastch;
+	return WS.lastch;
 }
 
 
@@ -2269,19 +2287,19 @@ rbiff(nbp)
  */
 
 int
-writebuf(fp, buf, len, lastchp)
-	FILE *fp;
+writebuf(WS, buf, len)
+	struct writestate *WS;
 	char *buf;
 	int len;
-	int *lastchp;
 {
-	register char *cp, *s;
+	register char *cp;
 	register int n;
 	int wlen, tlen, i = 0;
 	register char expect;
+	char *fromp;
+#if 0
 	static char save = '\0';
 	static char frombuf[8];
-	static char *fromp = NULL;
 
 	if (buf == NULL) {	/* magic initialization */
 	  save = mmdf_mode ? 0 : 'F';
@@ -2289,36 +2307,37 @@ writebuf(fp, buf, len, lastchp)
 	  fromp = frombuf;
 	  return 0;
 	}
-
+#endif
 	/* -------------------------------------------- */
 	/* Non-MIME processing				*/
 
-	expect = save;
+	expect = WS->expect;
+	fromp  = WS->fromp;
 
-	*lastchp = buf[len-1];
+	WS->lastch = buf[len-1];
 
-	for (cp = buf, n = len, tlen = 0, s = NULL; n > 0; --n, ++cp) {
+	for (cp = buf, n = len, tlen = 0; n > 0; --n, ++cp) {
 	  register int c = *cp;
 	  ++tlen;
 
 	  if (c == '\n') {
 	    expect = mmdf_mode ? 0 : 'F';
-	    frombuf[0] = 0;
-	    fromp = frombuf;
-	    if (putc(c,fp) == EOF)
+	    fromp = WS->frombuf;
+	    *fromp = 0;
+	    if (putc(c,WS->fp) == EOF)
 	      { tlen = -1; break; }
 	  } else if (expect != '\0') {
 	    if (c == expect) {
 	      *fromp++ = c;
 	      *fromp   = 0;
 	      switch (expect) {
-	        case 'F':	expect = 'r'; s = cp; break;
+	        case 'F':	expect = 'r'; break;
 		case 'r':	expect = 'o'; break;
 		case 'o':	expect = 'm'; break;
 		case 'm':	expect = ' '; break;
 		case ' ':
 		  /* Write the separator, and the word.. */
-		  if (fwrite(">From ", 6, 1, fp) == 0 || ferror(fp))
+		  if (fwrite(">From ", 6, 1, WS->fp) == 0 || ferror(WS->fp))
 		    { tlen = -1; break; }
 		  /* anticipate future instances */
 		  expect = '\0';
@@ -2326,39 +2345,37 @@ writebuf(fp, buf, len, lastchp)
 		}
 	    } else {
 	      expect = '\0';
-	      fromp = frombuf;
-	      while (*fromp) {
-		if (putc(*fromp,fp) == EOF)
-		  { tlen = -1; break; }
-		++fromp;
-	      }
-	      frombuf[0] = 0;
-	      if (putc(c,fp) == EOF)
+	      if (WS->frombuf[0] != 0 &&
+		  fputs(WS->frombuf, WS->fp) < 0)
+		{ tlen = -1; break; }
+	      WS->frombuf[0] = 0;
+	      fromp = WS->frombuf;
+	      if (putc(c,WS->fp) == EOF)
 		{ tlen = -1; break; }
 	    }
-	  } else {
-	    expect = '\0';
-	    if (putc(c,fp) == EOF)
+	  } else { /* expect == '\0'; */
+	    if (putc(c,WS->fp) == EOF)
 	      { tlen = -1; break; }
 	  }
 	}
+	WS->expect = expect;
+	WS->fromp  = fromp;
 	if (tlen >= 0) return len; /* It worked ok */
 	return -1;	/* errno is correct if tlen == -1 ! */
 }
 
 int
-writemimeline(fp, buf, len, lastchp)
-	FILE *fp;
+writemimeline(WS, buf, len)
+	struct writestate *WS;
 	char *buf;
 	int len;
-	int *lastchp;
 {
 	register char *cp, *s;
 	register int n;
 	int wlen, tlen, i = 0;
+	char *buf2;
+#if 0
 	static char save = '\0';
-	static char frombuf[8];
-	static char *fromp = NULL;
 
 	static char *buf2 = NULL;
 	static int buf2len = 0;
@@ -2369,20 +2386,20 @@ writemimeline(fp, buf, len, lastchp)
 	  fromp = frombuf;
 	  return 0;
 	}
-
+#endif
 #ifdef	USE_ALLOCA
-	buf2 = alloca(len+1);
+	WS->buf2 = alloca(len+1);
 #else
-	if (buf2 == NULL) {
-	  buf2 = emalloc(len+1);
-	  buf2len = len;
+	if (WS->buf2 == NULL) {
+	  WS->buf2 = emalloc(len+1);
+	  WS->buf2len = len;
 	}
-	if (buf2len < len) {
-	  free(buf2);
-	  buf2 = emalloc(len+1);
-	  buf2len = len;
+	if (WS->buf2len < len) {
+	  WS->buf2 = realloc(WS->buf2, len+1);
+	  WS->buf2len = len;
 	}
 #endif
+	buf2 = WS->buf2;
 
 	/* ------------------------------------------------------------ */
 	/* MIME format processing, the input is LINE BY LINE */
@@ -2391,8 +2408,8 @@ writemimeline(fp, buf, len, lastchp)
 	    strncmp(buf+2,mime_boundary,mime_boundary_len) == 0) {
 	  /* ------------------------------------------------------------ */
 	  /* XX: process MIME boundary! */
-	  *lastchp = buf[len-1];
-	  return fwrite(buf, 1, len, fp);
+	  WS->lastch = buf[len-1];
+	  return fwrite(buf, 1, len, WS->fp);
 
 	} else if (convert_qp) {
 	  /* ------------------------------------------------------------ */
@@ -2434,11 +2451,12 @@ writemimeline(fp, buf, len, lastchp)
 	  }
 	  /* Uhh... Stored the conversion result into buf2[], length  tlen */
 	  if (!mmdf_mode && buf2[0] == 'F' && strncmp(buf2,"From ",5)==0)
-	    if (putc('>',fp) == EOF)
-	      return -1;
+	    if (WS->lastch >= 256 || WS->lastch == '\n')
+	      if (putc('>',WS->fp) == EOF)
+		return -1;
 	  if (tlen > 0) {
-	    i = fwrite(buf2, 1, tlen, fp);
-	    *lastchp = buf2[tlen-1];
+	    i = fwrite(buf2, 1, tlen, WS->fp);
+	    WS->lastch = buf2[tlen-1];
 	  } else
 	    i = 0;
 	  if (i != tlen) return -1;
@@ -2448,9 +2466,11 @@ writemimeline(fp, buf, len, lastchp)
 	/* ------------------------------------------------------------ */
 	/* Well, no other processings known.. */
 	if (!mmdf_mode && buf[0] == 'F' && strncmp(buf,"From ",5)==0)
-	  putc('>',fp);
-	*lastchp = buf[len-1];
-	return fwrite(buf, 1, len, fp);
+	  if (WS->lastch >= 256 || WS->lastch == '\n')
+	    if (putc('>',WS->fp) == EOF)
+	      return -1;
+	WS->lastch = buf[len-1];
+	return fwrite(buf, 1, len, WS->fp);
 }
 
 
